@@ -70,7 +70,6 @@ class WorkAreaManager;
 namespace support {
 class DocFileName;
 class FileName;
-class FileNamePairList;
 } // namespace support
 
 namespace graphics {
@@ -158,9 +157,9 @@ public:
 	/// Destructor
 	~Buffer();
 
-	/// Clones the entire structure of which this Buffer is part, starting
-	/// with the master and cloning all the children, too.
-	Buffer * cloneFromMaster() const;
+	/// Clones the entire structure of which this Buffer is part, 
+	/// cloning all the children, too.
+	Buffer * cloneWithChildren() const;
 	/// Just clones this single Buffer. For autosave.
 	Buffer * cloneBufferOnly() const;
 	///
@@ -416,7 +415,7 @@ public:
 	 *  output in the respective BibTeX/Biblatex macro
 	 */
 	std::vector<docstring> const prepareBibFilePaths(OutputParams const &,
-				    support::FileNamePairList const bibfilelist,
+				    const docstring_list & bibfilelist,
 				    bool const extension = true) const;
 
 	/** Returns the path where a local layout file lives.
@@ -510,8 +509,8 @@ public:
 	/// Calling this method invalidates the cache and so requires a
 	/// re-read.
 	void invalidateBibinfoCache() const;
-	/// This invalidates the cache of files we need to check.
-	void invalidateBibfileCache() const;
+	/// Clear the bibfiles cache
+	void clearBibFileCache() const;
 	/// Updates the cached bibliography information, checking first to see
 	/// whether the cache is valid. If so, we do nothing. If not, then we
 	/// reload all the BibTeX info.
@@ -779,6 +778,10 @@ public:
 	void setChangesPresent(bool) const;
 	bool areChangesPresent() const;
 	void updateChangesPresent() const;
+	///
+	void registerBibfiles(docstring_list const & bf) const;
+	///
+	support::FileName getBibfilePath(docstring const & bibid) const;
 
 private:
 	friend class MarkAsExporting;
@@ -800,8 +803,8 @@ private:
 	void updateBibfilesCache(UpdateScope scope = UpdateMaster) const;
 	/// Return the list with all bibfiles in use (including bibfiles
 	/// of loaded child documents).
-	support::FileNamePairList const &
-		getBibfilesCache(UpdateScope scope = UpdateMaster) const;
+	docstring_list const &
+		getBibfiles(UpdateScope scope = UpdateMaster) const;
 	///
 	void collectChildren(ListOfBuffers & children, bool grand_children) const;
 
