@@ -39,12 +39,11 @@ class Bullet;
 class DocumentClass;
 class Encoding;
 class Font;
-class HSpace;
 class IndicesList;
 class Language;
-class LatexFeatures;
 class LayoutFile;
 class LayoutFileIndex;
+class Length;
 class Lexer;
 class PDFOptions;
 class Spacing;
@@ -94,20 +93,39 @@ public:
 	bool hasClassDefaults() const;
 
 	///
-	HSpace const & getIndentation() const;
+	Length const & getParIndent() const;
 	///
-	void setIndentation(HSpace const & indent);
+	void setParIndent(Length const & indent);
 	///
 	VSpace const & getDefSkip() const;
 	///
 	void setDefSkip(VSpace const & vs);
 
+	///
+	Length const & getMathIndent() const;
+	///
+	void setMathIndent(Length const & indent);
+
+	/// Whether formulas are indented
+	bool is_math_indent;
+
+
+	enum  MathNumber { DEFAULT, LEFT, RIGHT };
+	/// number formulas on left/right/default
+	MathNumber math_numbering_side;
+
+	/// Convenience function for display: like math_number, but
+	/// DEFAULT is replaced by the best guess we have.
+	MathNumber getMathNumber() const;
+
 	/** Whether paragraphs are separated by using a indent like in
 	 *  articles or by using a little skip like in letters.
 	 */
 	ParagraphSeparation paragraph_separation;
 	///
-	InsetQuotes::QuoteLanguage quotes_language;
+	InsetQuotesParams::QuoteStyle quotes_style;
+	///
+	bool dynamic_quotes;
 	///
 	std::string fontsize;
 	/// Get the LayoutFile this document is using.
@@ -143,7 +161,7 @@ public:
 			{ return removed_modules_; }
 	///
 	/// Add a module to the list of modules in use. This checks only that the
-	/// module is not already in the list, so use layoutModuleCanBeAdeed first 
+	/// module is not already in the list, so use layoutModuleCanBeAdeed first
 	/// if you want to check for compatibility.
 	/// \return true if module was successfully added.
 	bool addLayoutModule(std::string const & modName);
@@ -160,9 +178,9 @@ public:
 	/// Clear the removed module list
 	void clearRemovedModules() { removed_modules_.clear(); }
 	/// Get the local layouts
-	std::string getLocalLayout(bool) const;
+	docstring getLocalLayout(bool forced) const;
 	/// Set the local layouts
-	void setLocalLayout(std::string const &, bool);
+	void setLocalLayout(docstring const & layout, bool forced);
 
 	/// returns \c true if the buffer contains a LaTeX document
 	bool isLatex() const;
@@ -202,7 +220,7 @@ public:
 	Font const getFont() const;
 
 	/// translate quote style string to enum value
-	InsetQuotes::QuoteLanguage getQuoteStyle(std::string const & qs) const;
+	InsetQuotesParams::QuoteStyle getQuoteStyle(std::string const & qs) const;
 
 	/* these are for the PaperLayout */
 	/// the papersize
@@ -277,6 +295,10 @@ public:
 	int fontsTypewriterScale() const { return fonts_typewriter_scale[useNonTeXFonts]; }
 	/// the font used by the CJK command
 	std::string fonts_cjk;
+	/// use LaTeX microtype package
+	bool use_microtype;
+	/// use font ligatures for en- and em-dashes
+	bool use_dash_ligatures;
 	///
 	Spacing & spacing();
 	Spacing const & spacing() const;
@@ -318,7 +340,7 @@ public:
 	///
 	std::string origin;
 	///
-	std::string preamble;
+	docstring preamble;
 	///
 	std::string options;
 	/// use the class options defined in the layout?
@@ -375,11 +397,17 @@ public:
 	void use_package(std::string const & p, Package u);
 	/// All packages that can be switched on or off
 	static std::map<std::string, std::string> const & auto_packages();
+	/// Do we use the bibtopic package?
+	bool useBibtopic() const;
 	/// Split bibliography?
-	bool use_bibtopic;
+	bool splitbib() const { return use_bibtopic; }
+	/// Set split bibliography
+	void splitbib(bool const b) { use_bibtopic = b; }
+	/// Do we have multiple bibliographies (by chapter etc.)?
+	std::string multibib;
 	/// Split the index?
 	bool use_indices;
-	/// Save trensient properties?
+	/// Save transient properties?
 	bool save_transient_properties;
 	/// revision tracking for this buffer ? (this is a transient property)
 	bool track_changes;
@@ -402,7 +430,7 @@ public:
 	AuthorMap author_map_;
 
 	/// the buffer's active font encoding
-	std::string const font_encoding() const;
+	std::string const main_font_encoding() const;
 	/// all font encodings requested by the prefs/document/main language.
 	/// This does NOT include font encodings required by secondary languages
 	std::vector<std::string> const font_encodings() const;
@@ -459,12 +487,27 @@ public:
 	/// the available citation styles
 	std::vector<CitationStyle> citeStyles() const;
 
-	/// the default BibTeX style file for the document
-	std::string biblio_style;
-	/// the default BibTeX style file from the TextClass
+	/// Return the actual bibtex command (lyxrc or buffer param)
+	std::string const bibtexCommand() const;
+
+	/// Are we using biblatex?
+	bool useBiblatex() const;
+
+	/// Set the default BibTeX style file for the document
+	void setDefaultBiblioStyle(std::string const & s){ biblio_style = s; }
+	/// Get the default BibTeX style file from the TextClass
 	std::string const & defaultBiblioStyle() const;
 	/// whether the BibTeX style supports full author lists
 	bool const & fullAuthorList() const;
+	/// Check if a citation style is an alias to another style
+	std::string getCiteAlias(std::string const & s) const;
+
+	/// Options of the bibiography package
+	std::string biblio_opts;
+	/// The biblatex bibliography style
+	std::string biblatex_bibstyle;
+	/// The biblatex citation style
+	std::string biblatex_citestyle;
 
 	/// options for pdf output
 	PDFOptions & pdfoptions();
@@ -492,12 +535,16 @@ public:
 	std::string html_latex_end;
 	///
 	bool html_css_as_file;
+	/// allow the LaTeX backend to run external programs
+	bool shell_escape;
 	/// generate output usable for reverse/forward search
 	bool output_sync;
 	/// custom LaTeX macro from user instead our own
 	std::string output_sync_macro;
 	/// use refstyle? or prettyref?
 	bool use_refstyle;
+	/// use minted? or listings?
+	bool use_minted;
 
 	/// Return true if language could be set to lang,
 	/// otherwise return false and do not change language
@@ -532,6 +579,10 @@ private:
 	LayoutModuleList cite_engine_;
 	/// the type of cite engine (authoryear or numerical)
 	CiteEngineType cite_engine_type_;
+	/// the default BibTeX style file for the document
+	std::string biblio_style;
+	/// Split bibliography?
+	bool use_bibtopic;
 	///
 	DocumentClassPtr doc_class_;
 	///
@@ -540,9 +591,9 @@ private:
 	/// the user has chosen not to use
 	std::list<std::string> removed_modules_;
 	/// The local layouts without the forced ones
-	std::string local_layout_;
+	docstring local_layout_;
 	/// Forced local layouts only for reading (use getLocalLayout() instead)
-	std::string forced_local_layout_;
+	docstring forced_local_layout_;
 
 	/// the list of included children (for includeonly)
 	std::list<std::string> included_children_;
