@@ -476,6 +476,14 @@ void BufferView::processUpdateFlags(Update::flags flags)
 	if (flags == Update::None)
 		return;
 
+	/* FIXME We would like to avoid doing this here, since it is very
+	 * expensive and is called in updateBuffer already. However, even
+	 * inserting a plain character can invalidate the overly fragile
+	 * tables of child documents built by updateMacros. Some work is
+	 * needed to avoid doing that when not necessary.
+	 */
+	buffer_.updateMacros();
+
 	// SinglePar is ignored for now (this should probably change). We
 	// set it ourselves below, at the price of always rebreaking the
 	// paragraph at cursor. This can be expensive for large tables.
@@ -844,7 +852,7 @@ bool BufferView::moveToPosition(pit_type bottom_pit, pos_type bottom_pos,
 		d->cursor_.setCurrentFont();
 		// Do not forget to reset the anchor (see #9912)
 		d->cursor_.resetAnchor();
-		processUpdateFlags(Update::FitCursor);
+		processUpdateFlags(Update::Force | Update::FitCursor);
 	}
 
 	return success;
@@ -1664,7 +1672,6 @@ void BufferView::dispatch(FuncRequest const & cmd, DispatchResult & dr)
 						BIBTEX_CODE);
 		if (inset) {
 			if (inset->addDatabase(cmd.argument())) {
-				buffer_.invalidateBibfileCache();
 				dr.forceBufferUpdate();
 			}
 		}
@@ -1677,10 +1684,8 @@ void BufferView::dispatch(FuncRequest const & cmd, DispatchResult & dr)
 		InsetBibtex * inset = getInsetByCode<InsetBibtex>(tmpcur,
 						BIBTEX_CODE);
 		if (inset) {
-			if (inset->delDatabase(cmd.argument())) {
-				buffer_.invalidateBibfileCache();
+			if (inset->delDatabase(cmd.argument()))
 				dr.forceBufferUpdate();
-			}
 		}
 		break;
 	}
