@@ -453,19 +453,6 @@ int Cursor::currentMode()
 }
 
 
-bool Cursor::inCoordCache() const
-{
-	// the root inset is not in cache, but we do not need it.
-	if (depth() == 1)
-		return true;
-	CoordCache::Insets const & icache = bv_->coordCache().getInsets();
-	for (size_t i = 1 ; i < depth() ; ++i)
-		if (!icache.has(&(*this)[i].inset()))
-			return false;
-	return true;
-}
-
-
 void Cursor::getPos(int & x, int & y) const
 {
 	Point p = bv().getPos(*this);
@@ -1413,7 +1400,7 @@ bool Cursor::down()
 }
 
 
-bool Cursor::macroModeClose()
+bool Cursor::macroModeClose(bool cancel)
 {
 	if (!inMacroMode())
 		return false;
@@ -1425,14 +1412,15 @@ bool Cursor::macroModeClose()
 	--pos();
 	cell().erase(pos());
 
-	// do nothing if the macro name is empty
-	if (s == "\\")
-		return false;
-
 	// trigger updates of macros, at least, if no full
 	// updates take place anyway
 	screenUpdateFlags(Update::Force);
 
+	// do nothing if the macro name is empty
+	if (s == "\\" || cancel) {
+		return false;
+	}
+
 	docstring const name = s.substr(1);
 	InsetMathNest * const in = inset().asInsetMath()->asNestInset();
 	if (in && in->interpretString(*this, s))
