@@ -215,8 +215,6 @@ public:
 	/// are we entering a macro name?
 	bool & macromode() { return macromode_; }
 
-	/// returns true when all insets in cursor stack are in cache
-	bool inCoordCache() const;
 	/// returns x,y position
 	void getPos(int & x, int & y) const;
 	/// return logical positions between which the cursor is situated
@@ -502,9 +500,9 @@ public:
 	void setScreenPos(int x, int y);
 	/// current offset in the top cell
 
-	/// interpret name a name of a macro. Returns true if
-	/// something got inserted.
-	bool macroModeClose();
+	/// interpret name of a macro or ditch it if \c cancel is true.
+	/// Returns true if something got inserted.
+	bool macroModeClose(bool cancel = false);
 	/// are we currently typing the name of a macro?
 	bool inMacroMode() const;
 	/// get access to the macro we are currently typing
