@@ -42,7 +42,7 @@ public:
 		zipped_native = 8
 	};
 	///
-	Format(std::string const & n, std::string const & e, std::string const & p,
+	Format(std::string const & n, std::string const & e, docstring const & p,
 	       std::string const & s, std::string const & v, std::string const & ed,
 	       std::string const & m, int);
 	///
@@ -68,9 +68,9 @@ public:
 	///
 	void setExtensions(std::string const & v);
 	///
-	std::string const prettyname() const { return prettyname_; }
+	docstring const prettyname() const { return prettyname_; }
 	///
-	void setPrettyname(std::string const & v) { prettyname_ = v; }
+	void setPrettyname(docstring const & v) { prettyname_ = v; }
 	///
 	std::string const shortcut() const { return shortcut_; }
 	///
@@ -106,7 +106,7 @@ private:
 	/// Filename extensions, the first one being the default
 	std::vector<std::string> extension_list_;
 	/// Name presented to the user. Needs to be unique.
-	trivstring prettyname_;
+	trivdocstring prettyname_;
 	/// Keyboard shortcut for the View and Export menu.
 	trivstring shortcut_;
 	/*!
@@ -143,9 +143,9 @@ public:
 	///
 	typedef FormatList::const_iterator const_iterator;
 	///
-	Format const & get(FormatList::size_type i) const { return formatlist[i]; }
+	Format const & get(FormatList::size_type i) const { return formatlist_[i]; }
 	///
-	Format & get(FormatList::size_type i) { return formatlist[i]; }
+	Format & get(FormatList::size_type i) { return formatlist_[i]; }
 	/// \returns format named \p name if it exists, otherwise 0
 	Format const * getFormat(std::string const & name) const;
 	/*!
@@ -153,6 +153,7 @@ public:
 	 * fails, from file extension.
 	 * \returns file format if it could be found, otherwise an empty
 	 * string.
+	 * This function is expensive.
 	 */
 	std::string getFormatFromFile(support::FileName const & filename) const;
 	/// Finds a format from a file extension. Returns string() if not found.
@@ -175,7 +176,7 @@ public:
 	void add(std::string const & name);
 	///
 	void add(std::string const & name, std::string const & extensions,
-		 std::string const & prettyname, std::string const & shortcut,
+		 docstring const & prettyname, std::string const & shortcut,
 		 std::string const & viewer, std::string const & editor,
 		 std::string const & mime, int flags);
 	///
@@ -199,16 +200,16 @@ public:
 	///
 	std::string const extensions(std::string const & name) const;
 	///
-	const_iterator begin() const { return formatlist.begin(); }
+	const_iterator begin() const { return formatlist_.begin(); }
 	///
-	const_iterator end() const { return formatlist.end(); }
+	const_iterator end() const { return formatlist_.end(); }
 	///
-	bool empty() const { return formatlist.empty(); }
+	bool empty() const { return formatlist_.empty(); }
 	///
-	FormatList::size_type size() const { return formatlist.size(); }
+	FormatList::size_type size() const { return formatlist_.size(); }
 private:
 	///
-	FormatList formatlist;
+	FormatList formatlist_;
 };
 
 ///
@@ -216,10 +217,13 @@ std::string flavor2format(OutputParams::FLAVOR flavor);
 // Not currently used.
 // OutputParams::FLAVOR format2flavor(std::string fmt);
 
-extern Formats formats;
-
-extern Formats system_formats;
+/// The global instance.
+/// Implementation is in LyX.cpp.
+extern Formats & theFormats();
 
+/// The global copy after reading lyxrc.defaults.
+/// Implementation is in LyX.cpp.
+extern Formats & theSystemFormats();
 
 } // namespace lyx
 
