@@ -65,6 +65,10 @@ namespace lyx {
 static docstring const lyx_def = from_ascii(
 	"\\providecommand{\\LyX}{L\\kern-.1667em\\lower.25em\\hbox{Y}\\kern-.125emX\\@}");
 
+static docstring const lyx_rtl_def = from_ascii(
+	"\\let\\@@LyX\\LyX\n"
+	"\\def\\LyX{\\@ensure@LTR{\\@@LyX}}");
+
 static docstring const lyx_hyperref_def = from_ascii(
 	"\\providecommand{\\LyX}{\\texorpdfstring%\n"
 	"  {L\\kern-.1667em\\lower.25em\\hbox{Y}\\kern-.125emX\\@}\n"
@@ -908,6 +912,17 @@ void LaTeXFeatures::getFontEncodings(vector<string> & encodings) const
 		}
 }
 
+
+bool LaTeXFeatures::hasRTLLanguage() const
+{
+	if (params_.language->rightToLeft())
+		return true;
+	for (auto const & lang : UsedLanguages_)
+		if (lang->rightToLeft())
+			return true;
+	return false;
+}
+
 namespace {
 
 char const * simplefeatures[] = {
@@ -949,7 +964,6 @@ char const * simplefeatures[] = {
 	// "cancel",
 	"ascii",
 	"url",
-	"covington",
 	"csquotes",
 	"enumitem",
 	"endnotes",
@@ -974,7 +988,6 @@ char const * simplefeatures[] = {
 	"todonotes",
 	"forest",
 	"varwidth",
-	"footnote",
 	"tablefootnote",
 };
 
@@ -1098,6 +1111,13 @@ string const LaTeXFeatures::getPackages() const
 	// The rest of these packages are somewhat more complicated
 	// than those above.
 
+	if (mustProvide("footnote")) {
+		if (isRequired("hyperref"))
+			packages << "\\usepackage{footnotehyper}\n";
+		else
+			packages << "\\usepackage{footnote}\n";
+	}
+
 	// [pdf]lscape is used to rotate longtables
 	if (mustProvide("lscape")) {
 		if (runparams_.flavor == OutputParams::LATEX
@@ -1353,6 +1373,8 @@ TexString LaTeXFeatures::getMacros() const
 			macros << lyx_hyperref_def << '\n';
 		else
 			macros << lyx_def << '\n';
+		if (runparams_.use_polyglossia && hasRTLLanguage())
+			macros << lyx_rtl_def << '\n';
 	}
 
 	if (mustProvide("noun"))
