@@ -24,10 +24,11 @@ OutputParams::OutputParams(Encoding const * enc)
 	  encoding(enc), free_spacing(false), use_babel(false), use_polyglossia(false),
 	  use_indices(false), use_japanese(false), linelen(0), depth(0),
 	  exportdata(new ExportData), inDisplayMath(false), wasDisplayMath(false),
-	  inComment(false), inTableCell(NO), inFloat(NONFLOAT),
+	  inComment(false), openbtUnit(false), only_childbibs(false),
+	  inTableCell(NO), inFloat(NONFLOAT),
 	  inIndexEntry(false), inIPA(false), inDeletedInset(0),
 	  changeOfDeletedInset(Change::UNCHANGED),
-	  par_begin(0), par_end(0), lastid(-1), lastpos(-1), isLastPar(false),
+	  par_begin(0), par_end(0), lastid(-1), lastpos(0), isLastPar(false),
 	  dryrun(false), silent(false), pass_thru(false),
 	  html_disable_captions(false), html_in_par(false),
 	  html_make_pars(true), for_toc(false), for_tooltip(false),
