@@ -149,8 +149,8 @@ void RowPainter::paintForeignMark(Row::Element const & e) const
 		return;
 
 	int const desc = e.inset ? e.dim.descent() : 0;
-	int const y = yo_ + pi_.base.solidLineOffset()
-		+ desc + pi_.base.solidLineThickness() / 2;
+	int const y = yo_ + min(3 * pi_.base.solidLineOffset() / 2 + desc,
+	                        row_.descent() - 1);
 	pi_.pain.line(int(x_), y, int(x_ + e.full_width()), y, Color_language,
 	              Painter::line_solid, pi_.base.solidLineThickness());
 }
@@ -239,6 +239,21 @@ void RowPainter::paintStringAndSel(Row::Element const & e) const
 }
 
 
+void RowPainter::paintTextDecoration(Row::Element const & e) const
+{
+	// element selected?
+	bool const sel = (e.pos >= row_.sel_beg && e.endpos <= row_.sel_end)
+		|| pi_.selected;
+	FontInfo copy = e.font.fontInfo();
+	if (sel || e.change.changed()) {
+		Color const col = e.change.changed() ? e.change.color()
+		                                     : Color_selectiontext;
+		copy.setPaintColor(col);
+	}
+	pi_.pain.textDecoration(copy, int(x_), yo_, int(e.full_width()));
+}
+
+
 void RowPainter::paintChange(Row::Element const & e) const
 {
 	e.change.paintCue(pi_, x_, yo_, x_ + e.full_width(), e.font.fontInfo());
@@ -570,10 +585,11 @@ void RowPainter::paintText()
 
 		case Row::INSET:
 			paintInset(e);
+			paintTextDecoration(e);
 			break;
 
 		case Row::SPACE:
-			pi_.pain.textDecoration(e.font.fontInfo(), int(x_), yo_, int(e.full_width()));
+			paintTextDecoration(e);
 		}
 
 		// The line that indicates word in a different language
