@@ -968,14 +968,19 @@ void Text::insertChar(Cursor & cur, char_type c)
 	if (lyxrc.auto_number) {
 		static docstring const number_operators = from_ascii("+-/*");
 		static docstring const number_unary_operators = from_ascii("+-");
-		static docstring const number_separators = from_ascii(".,:");
 
+		// European Number Separators: comma, dot etc.
+		// European Number Terminators: percent, permille, degree, euro etc.
 		if (cur.current_font.fontInfo().number() == FONT_ON) {
 			if (!isDigitASCII(c) && !contains(number_operators, c) &&
-			    !(contains(number_separators, c) &&
+			    !(isEuropeanNumberSeparator(c) &&
 			      cur.pos() != 0 &&
 			      cur.pos() != cur.lastpos() &&
 			      tm.displayFont(pit, cur.pos()).fontInfo().number() == FONT_ON &&
+			      tm.displayFont(pit, cur.pos() - 1).fontInfo().number() == FONT_ON) &&
+			    !(isEuropeanNumberTerminator(c) &&
+			      cur.pos() != 0 &&
+			      tm.displayFont(pit, cur.pos()).fontInfo().number() == FONT_ON &&
 			      tm.displayFont(pit, cur.pos() - 1).fontInfo().number() == FONT_ON)
 			   )
 				number(cur); // Set current_font.number to OFF
@@ -993,7 +998,7 @@ void Text::insertChar(Cursor & cur, char_type c)
 				  ) {
 					setCharFont(pit, cur.pos() - 1, cur.current_font,
 						tm.font_);
-				} else if (contains(number_separators, c)
+				} else if (isEuropeanNumberSeparator(c)
 				     && cur.pos() >= 2
 				     && tm.displayFont(pit, cur.pos() - 2).fontInfo().number() == FONT_ON) {
 					setCharFont(pit, cur.pos() - 1, cur.current_font,
@@ -1476,7 +1481,7 @@ void Text::deleteWordForward(Cursor & cur, bool const force)
 		cursorForwardOneWord(cur);
 		cur.setSelection();
 		if (force || !cur.confirmDeletion()) {
-			cutSelection(cur, true, false);
+			cutSelection(cur, false);
 			cur.checkBufferStructure();
 		}
 	}
@@ -1494,7 +1499,7 @@ void Text::deleteWordBackward(Cursor & cur, bool const force)
 		cursorBackwardOneWord(cur);
 		cur.setSelection();
 		if (force || !cur.confirmDeletion()) {
-			cutSelection(cur, true, false);
+			cutSelection(cur, false);
 			cur.checkBufferStructure();
 		}
 	}
