@@ -1573,7 +1573,7 @@ void TextMetrics::deleteLineForward(Cursor & cur)
 		if (!cur.selection())
 			text_->deleteWordForward(cur);
 		else
-			cap::cutSelection(cur, true, false);
+			cap::cutSelection(cur, false);
 		cur.checkBufferStructure();
 	}
 }
@@ -1618,10 +1618,10 @@ int TextMetrics::leftMargin(pit_type const pit, pos_type const pos) const
 
 	int l_margin = 0;
 
-	if (text_->isMainText())
+	if (text_->isMainText()) {
 		l_margin += bv_->leftMargin();
-
-	l_margin += bfm.signedWidth(tclass.leftmargin());
+		l_margin += bfm.signedWidth(tclass.leftmargin());
+	}
 
 	int depth = par.getDepth();
 	if (depth != 0) {
