@@ -31,6 +31,8 @@
 #include "support/TempFile.h"
 
 #include <fstream>
+#include <iomanip>
+#include <sstream>
 
 using namespace std;
 using namespace lyx::support;
@@ -169,6 +171,8 @@ void RCS::scanMaster()
 	LYXERR(Debug::LYXVC, "LyXVC::RCS: scanMaster: " << master_);
 
 	ifstream ifs(master_.toFilesystemEncoding().c_str());
+	// limit the size of strings we read to avoid memory problems
+	ifs >> setw(65636);
 
 	string token;
 	bool read_enough = false;
@@ -283,7 +287,7 @@ LyXVC::CommandResult RCS::checkIn(string const & msg, string & log)
 
 bool RCS::checkInEnabled()
 {
-	return owner_ && !owner_->isReadonly();
+	return owner_ && !owner_->hasReadonlyFlag();
 }
 
 
@@ -325,7 +329,7 @@ string RCS::checkOut()
 
 bool RCS::checkOutEnabled()
 {
-	return owner_ && owner_->isReadonly();
+	return owner_ && owner_->hasReadonlyFlag();
 }
 
 
@@ -388,7 +392,7 @@ void RCS::undoLast()
 
 bool RCS::undoLastEnabled()
 {
-	return true;
+	return owner_->hasReadonlyFlag();
 }
 
 
@@ -426,7 +430,8 @@ string RCS::revisionInfo(LyXVC::RevisionInfo const info)
 			return rev_date_cache_;
 		case LyXVC::Time:
 			return rev_time_cache_;
-		default: ;
+		default:
+			break;
 	}
 
 	return string();
@@ -565,8 +570,10 @@ void CVS::scanMaster()
 		if (contains(line, tmpf)) {
 			// Ok extract the fields.
 			smatch sm;
-
-			regex_match(line, sm, reg);
+			if (!regex_match(line, sm, reg)) {
+				LYXERR(Debug::LYXVC, "\t  Cannot parse line. Skipping.");
+				continue;
+			}
 
 			//sm[0]; // whole matched string
 			//sm[1]; // filename
@@ -1096,7 +1103,8 @@ string CVS::revisionInfo(LyXVC::RevisionInfo const info)
 			return rev_date_cache_;
 		case LyXVC::Time:
 			return rev_time_cache_;
-		default: ;
+		default:
+			break;
 		}
 	}
 	return string();
@@ -1432,7 +1440,7 @@ bool SVN::fileLock(bool lock, FileName const & tmpf, string &status)
 		    + " > " + quoteName(tmpf.toFilesystemEncoding()),
 		    FileName(owner_->filePath()));
 
-	// Lock error messages go unfortunately on stderr and are unreachible this way.
+	// Lock error messages go unfortunately on stderr and are unreachable this way.
 	ifstream ifs(tmpf.toFilesystemEncoding().c_str());
 	string line;
 	while (ifs) {
@@ -1660,8 +1668,8 @@ string SVN::revisionInfo(LyXVC::RevisionInfo const info)
 			return rev_date_cache_;
 		case LyXVC::Time:
 			return rev_time_cache_;
-		default: ;
-
+		default:
+			break;
 	}
 
 	return string();
@@ -2129,8 +2137,8 @@ string GIT::revisionInfo(LyXVC::RevisionInfo const info)
 			return rev_date_cache_;
 		case LyXVC::Time:
 			return rev_time_cache_;
-		default: ;
-
+		default:
+			break;
 	}
 
 	return string();
