@@ -14,18 +14,14 @@
 
 #include "support/docstring.h"
 
+#include <memory>
+
 namespace lyx {
 
 class WordList {
-	// noncopyable because of pimpl
-	WordList(WordList const &);
-	WordList & operator=(WordList const &);
 public:
 	///
 	WordList();
-	///
-	~WordList();
-
 	///
 	docstring const & word(size_t idx) const;
 	///
@@ -34,15 +30,13 @@ public:
 	void insert(docstring const & w);
 	///
 	void remove(docstring const & w);
-	///
-	static void cleanupWordLists();
 
 private:
 	struct Impl;
-	Impl * d;
+	std::unique_ptr<Impl> d;
 };
 
-WordList * theWordList(std::string const & lang);
+WordList & theWordList(std::string const & lang);
 
 } // namespace lyx
 
