@@ -16,15 +16,14 @@
 #include "qt_helpers.h"
 
 #include "Dimension.h"
-#include "Language.h"
-#include "LyXRC.h"
-
-#include "insets/Inset.h"
 
 #include "support/convert.h"
 #include "support/lassert.h"
 #include "support/lyxlib.h"
 
+#define DISABLE_PMPROF
+#include "support/pmprof.h"
+
 #include <QByteArray>
 
 using namespace std;
@@ -43,7 +42,7 @@ uint qHash(lyx::docstring const & s)
 	                        s.size() * sizeof(lyx::docstring::value_type)));
 }
 
-}
+} // namespace std
 
 namespace lyx {
 namespace frontend {
@@ -86,7 +85,7 @@ inline QChar const ucs4_to_qchar(char_type const ucs4)
 	LATTEST(is_utf16(ucs4));
 	return QChar(static_cast<unsigned short>(ucs4));
 }
-} // anon namespace
+} // namespace
 
 
 GuiFontMetrics::GuiFontMetrics(QFont const & font)
@@ -178,8 +177,10 @@ int GuiFontMetrics::rbearing(char_type c) const
 
 int GuiFontMetrics::width(docstring const & s) const
 {
+	PROFILE_THIS_BLOCK(width);
 	if (strwidth_cache_.contains(s))
 		return strwidth_cache_[s];
+	PROFILE_CACHE_MISS(width);
 	/* For some reason QMetrics::width returns a wrong value with Qt5
 	 * with some arabic text. OTOH, QTextLayout is broken for single
 	 * characters with null width (like \not in mathed). Also, as a
@@ -199,11 +200,7 @@ int GuiFontMetrics::width(docstring const & s) const
 		tl.beginLayout();
 		QTextLine line = tl.createLine();
 		tl.endLayout();
-#if QT_VERSION >= 0x040700
 		w = iround(line.horizontalAdvance());
-#else
-		w = iround(line.naturalTextWidth());
-#endif
 	}
 	strwidth_cache_.insert(s, w, s.size() * sizeof(char_type));
 	return w;
@@ -232,11 +229,13 @@ shared_ptr<QTextLayout const>
 GuiFontMetrics::getTextLayout(docstring const & s, bool const rtl,
                               double const wordspacing) const
 {
+	PROFILE_THIS_BLOCK(getTextLayout);
 	docstring const s_cache =
 		s + (rtl ? "r" : "l") + convert<docstring>(wordspacing);
-	if (shared_ptr<QTextLayout const> ptl = qtextlayout_cache_[s_cache])
+	if (auto ptl = qtextlayout_cache_[s_cache])
 		return ptl;
-	shared_ptr<QTextLayout> const ptl = make_shared<QTextLayout>();
+	PROFILE_CACHE_MISS(getTextLayout);
+	auto const ptl = make_shared<QTextLayout>();
 	ptl->setCacheEnabled(true);
 	ptl->setText(toqstr(s));
 	QFont copy = font_;
@@ -323,6 +322,26 @@ int GuiFontMetrics::x2pos(docstring const & s, int & x, bool const rtl,
 }
 
 
+int GuiFontMetrics::countExpanders(docstring const & str) const
+{
+	// Numbers of characters that are expanded by inter-word spacing.  These
+	// characters are spaces, except for characters 09-0D which are treated
+	// specially.  (From a combination of testing with the notepad found in qt's
+	// examples, and reading the source code.)  In addition, consecutive spaces
+	// only count as one expander.
+	bool wasspace = false;
+	int nexp = 0;
+	for (char_type c : str)
+		if (c > 0x0d && QChar(c).isSpace()) {
+			if (!wasspace) {
+				++nexp;
+				wasspace = true;
+			}
+		} else
+			wasspace = false;
+	return nexp;
+}
+
 
 pair<int, int>
 GuiFontMetrics::breakAt_helper(docstring const & s, int const x,
@@ -368,13 +387,9 @@ GuiFontMetrics::breakAt_helper(docstring const & s, int const x,
 	line.setLineWidth(x);
 	tl.createLine();
 	tl.endLayout();
-#if QT_VERSION >= 0x040700
 	int const line_wid = iround(line.horizontalAdvance());
-#else
-	int const line_wid = iround(line.naturalTextWidth());
-#endif
 	if ((force && line.textLength() == offset) || line_wid > x)
-		return make_pair(-1, -1);
+		return {-1, -1};
 	/* Since QString is UTF-16 and docstring is UCS-4, the offsets may
 	 * not be the same when there are high-plan unicode characters
 	 * (bug #10443).
@@ -398,12 +413,13 @@ GuiFontMetrics::breakAt_helper(docstring const & s, int const x,
 		--len;
 	LASSERT(len > 0 || qlen == 0, /**/);
 #endif
-	return make_pair(len, line_wid);
+	return {len, line_wid};
 }
 
 
 bool GuiFontMetrics::breakAt(docstring & s, int & x, bool const rtl, bool const force) const
 {
+	PROFILE_THIS_BLOCK(breakAt);
 	if (s.empty())
 		return false;
 
@@ -414,6 +430,7 @@ bool GuiFontMetrics::breakAt(docstring & s, int & x, bool const rtl, bool const
 	if (breakat_cache_.contains(s_cache))
 		pp = breakat_cache_[s_cache];
 	else {
+		PROFILE_CACHE_MISS(breakAt);
 		pp = breakAt_helper(s, x, rtl, force);
 		breakat_cache_.insert(s_cache, pp, s_cache.size() * sizeof(char_type));
 	}
@@ -428,19 +445,20 @@ bool GuiFontMetrics::breakAt(docstring & s, int & x, bool const rtl, bool const
 void GuiFontMetrics::rectText(docstring const & str,
 	int & w, int & ascent, int & descent) const
 {
-	static int const d = Inset::TEXT_TO_INSET_OFFSET / 2;
+	// FIXME: let offset depend on font (this is Inset::TEXT_TO_OFFSET)
+	int const offset = 4;
 
-	w = width(str) + Inset::TEXT_TO_INSET_OFFSET;
-	ascent = metrics_.ascent() + d;
-	descent = metrics_.descent() + d;
+	w = width(str) + offset;
+	ascent = metrics_.ascent() + offset / 2;
+	descent = metrics_.descent() + offset / 2;
 }
 
 
-void GuiFontMetrics::buttonText(docstring const & str,
+void GuiFontMetrics::buttonText(docstring const & str, const int offset,
 	int & w, int & ascent, int & descent) const
 {
 	rectText(str, w, ascent, descent);
-	w += Inset::TEXT_TO_INSET_OFFSET;
+	w += offset;
 }
 
 
