@@ -897,7 +897,23 @@ void GuiWorkArea::wheelEvent(QWheelEvent * ev)
 {
 	// Wheel rotation by one notch results in a delta() of 120 (see
 	// documentation of QWheelEvent)
+	// But first we have to ignore horizontal scroll events.
+#if QT_VERSION < 0x050000
+	if (ev->orientation() == Qt::Horizontal) {
+		ev->accept();
+		return;
+	}
 	double const delta = ev->delta() / 120.0;
+#else
+	QPoint const aDelta = ev->angleDelta();
+	// skip horizontal wheel event
+	if (abs(aDelta.x()) > abs(aDelta.y())) {
+		ev->accept();
+		return;
+	}
+	double const delta = aDelta.y() / 120.0;
+#endif
+
 	bool zoom = false;
 	switch (lyxrc.scroll_wheel_zoom) {
 	case LyXRC::SCROLL_WHEEL_ZOOM_CTRL:
@@ -1252,6 +1268,8 @@ void GuiWorkArea::paintEvent(QPaintEvent * ev)
 		// backing store has to be copied to screen (this is a no-op
 		// except on macOS).
 		d->updateScreen(ev->rect());
+		// Ignore this paint event, but request a new one for later.
+		viewport()->update(ev->rect());
 		ev->accept();
 		return;
 	}
@@ -1290,12 +1308,13 @@ void GuiWorkArea::inputMethodEvent(QInputMethodEvent * e)
 
 	// insert the processed text in the document (handles undo)
 	if (!e->commitString().isEmpty()) {
-		d->buffer_view_->cursor().beginUndoGroup();
-		d->buffer_view_->cursor().insert(qstring_to_ucs4(e->commitString()));
+		FuncRequest cmd(LFUN_SELF_INSERT,
+		                qstring_to_ucs4(e->commitString()),
+		                FuncRequest::KEYBOARD);
+		dispatch(cmd);
+		// FIXME: this is supposed to remove traces from preedit
+		// string. Can we avoid calling it explicitely?
 		d->buffer_view_->updateMetrics();
-		d->buffer_view_->cursor().endUndoGroup();
-		d->updateCaretGeometry();
-		viewport()->update();
 	}
 
 	// Hide the caret during the test transformation.
