@@ -44,7 +44,6 @@ public:
 };
 
 
-// FIXME THREAD
 Cache & Cache::get()
 {
 	// Now return the cache
@@ -72,12 +71,12 @@ vector<string> const & Cache::loadableFormats() const
 		return fmts;
 
 	// The formats recognised by LyX
-	Formats::const_iterator begin = formats.begin();
-	Formats::const_iterator end   = formats.end();
+	Formats::const_iterator begin = theFormats().begin();
+	Formats::const_iterator end   = theFormats().end();
 
 	// The formats natively loadable.
 	vector<string> nformat = frontend::loadableImageFormats();
-	
+
 	vector<string>::const_iterator it = nformat.begin();
 	for (; it != nformat.end(); ++it) {
 		for (Formats::const_iterator fit = begin; fit != end; ++fit) {
@@ -94,18 +93,15 @@ vector<string> const & Cache::loadableFormats() const
 		vector<string>::const_iterator fbegin = fmts.begin();
 		vector<string>::const_iterator fend = fmts.end();
 		for (vector<string>::const_iterator fit = fbegin; fit != fend; ++fit) {
-			if (fit != fbegin)
-				LYXERR(Debug::GRAPHICS, ", ");
-			LYXERR(Debug::GRAPHICS, *fit);
+			LYXERR(Debug::GRAPHICS, *fit << ',');
 		}
-		LYXERR(Debug::GRAPHICS, '\n');
 	}
 
 	return fmts;
 }
 
 
-void Cache::add(FileName const & file) const
+void Cache::add(FileName const & file, FileName const & doc_file) const
 {
 	// Is the file in the cache already?
 	if (inCache(file)) {
@@ -114,7 +110,7 @@ void Cache::add(FileName const & file) const
 		return;
 	}
 
-	pimpl_->cache[file] = ItemPtr(new CacheItem(file));
+	pimpl_->cache[file] = ItemPtr(new CacheItem(file, doc_file));
 }
 
 
