@@ -42,9 +42,6 @@ using namespace lyx::support;
 
 namespace lyx {
 
-ParamInfo InsetCitation::param_info_;
-
-
 InsetCitation::InsetCitation(Buffer * buf, InsetCommandParams const & p)
 	: InsetCommand(buf, p)
 {
@@ -62,8 +59,16 @@ InsetCitation::~InsetCitation()
 }
 
 
+// May well be over-ridden when session settings are loaded
+// in GuiCitation. Unfortunately, that will not happen until
+// such a dialog is created.
+bool InsetCitation::last_literal = true;
+
+
 ParamInfo const & InsetCitation::findInfo(string const & /* cmdName */)
 {
+	static ParamInfo param_info_;
+
 	// standard cite does only take one argument, but biblatex, jurabib
 	// and natbib extend this to two arguments, so
 	// we have to allow both here. InsetCitation takes care that
@@ -384,9 +389,12 @@ docstring InsetCitation::complexLabel(bool for_xhtml) const
 	buffer().params().documentClass().addCiteMacro("!textafter", to_utf8(after));
 	*/
 	docstring label;
-	vector<docstring> keys = getVectorFromString(key);
+	// we only really want the last 'false', to suppress trimming, but
+	// we need to give the other defaults, too, to set it.
+	vector<docstring> keys =
+		getVectorFromString(key, from_ascii(","), false, false);
 	CitationStyle cs = getCitationStyle(buffer().masterParams(),
-					    cite_type, buffer().masterParams().citeStyles());
+			cite_type, buffer().masterParams().citeStyles());
 	bool const qualified = cs.hasQualifiedList
 		&& (keys.size() > 1
 		    || !getParam("pretextlist").empty()
@@ -422,8 +430,7 @@ docstring InsetCitation::basicLabel(bool for_xhtml) const
 	do {
 		// if there is no comma, then everything goes into key
 		// and keys will be empty.
-		keys = trim(split(keys, key, ','));
-		key = trim(key);
+		keys = split(keys, key, ',');
 		if (!label.empty())
 			label += ", ";
 		label += wrapCitation(key, key, for_xhtml);
@@ -545,7 +552,10 @@ void InsetCitation::forOutliner(docstring & os, size_t const, bool const) const
 // engine, e.g. \cite[]{} for the basic engine.
 void InsetCitation::latex(otexstream & os, OutputParams const & runparams) const
 {
-	BiblioInfo const & bi = buffer().masterBibInfo();
+	// When this is a child compiled on its own, we use the childs
+	// own bibinfo, else the master's
+	BiblioInfo const & bi = runparams.is_child
+			? buffer().masterBibInfo() : buffer().bibInfo();
 	docstring const key = getParam("key");
 	// "keyonly" command: output the plain key and stop.
 	if (getCmdName() == "keyonly") {
@@ -577,10 +587,11 @@ void InsetCitation::latex(otexstream & os, OutputParams const & runparams) const
 	if (qualified)
 		os << "s";
 
+	ParamInfo const & pinfo = findInfo(string());
 	docstring before = params().prepareCommand(runparams, getParam("before"),
-						   param_info_["before"].handling());
+						   pinfo["before"].handling());
 	docstring after = params().prepareCommand(runparams, getParam("after"),
-						   param_info_["after"].handling());
+						   pinfo["after"].handling());
 	if (!before.empty() && cs.textBefore) {
 		if (qualified)
 			os << '(' << protectArgument(before, '(', ')')
@@ -604,9 +615,9 @@ void InsetCitation::latex(otexstream & os, OutputParams const & runparams) const
 			map<docstring, docstring> posts = getQualifiedLists(getParam("posttextlist"));
 			for (docstring const & k: keys) {
 				docstring bef = params().prepareCommand(runparams, pres[k],
-									param_info_["pretextlist"].handling());
-				docstring aft  = params().prepareCommand(runparams, posts[k],
-									 param_info_["posttextlist"].handling());
+				                   pinfo["pretextlist"].handling());
+				docstring aft = params().prepareCommand(runparams, posts[k],
+				                   pinfo["posttextlist"].handling());
 				if (!bef.empty())
 					os << '[' << protectArgument(bef)
 					   << "][" << protectArgument(aft) << ']';
