@@ -29,7 +29,7 @@ namespace lyx {
 using support::token;
 
 InsetFootlike::InsetFootlike(Buffer * buf)
-	: InsetCollapsable(buf)
+	: InsetCollapsible(buf)
 {}
 
 
@@ -37,7 +37,7 @@ void InsetFootlike::metrics(MetricsInfo & mi, Dimension & dim) const
 {
 	FontInfo tmpfont = mi.base.font;
 	mi.base.font = mi.base.bv->buffer().params().getFont().fontInfo();
-	InsetCollapsable::metrics(mi, dim);
+	InsetCollapsible::metrics(mi, dim);
 	mi.base.font = tmpfont;
 }
 
@@ -46,7 +46,7 @@ void InsetFootlike::draw(PainterInfo & pi, int x, int y) const
 {
 	FontInfo tmpfont = pi.base.font;
 	pi.base.font = pi.base.bv->buffer().params().getFont().fontInfo();
-	InsetCollapsable::draw(pi, x, y);
+	InsetCollapsible::draw(pi, x, y);
 	pi.base.font = tmpfont;
 }
 
@@ -55,7 +55,7 @@ void InsetFootlike::write(ostream & os) const
 {
 	// The layoutName may contain a "InTitle" qualifier
 	os << to_utf8(token(layoutName(), char_type(':'), 0)) << "\n";
-	InsetCollapsable::write(os);
+	InsetCollapsible::write(os);
 }
 
 
@@ -63,7 +63,7 @@ bool InsetFootlike::insetAllowed(InsetCode code) const
 {
 	if (code == FOOT_CODE || code == MARGIN_CODE || code == FLOAT_CODE)
 		return false;
-	return InsetCollapsable::insetAllowed(code);
+	return InsetCollapsible::insetAllowed(code);
 }
 
 
