@@ -197,11 +197,6 @@ InsetInclude::InsetInclude(InsetInclude const & other)
 
 InsetInclude::~InsetInclude()
 {
-	if (isBufferLoaded())
-		/* We do not use buffer() because Coverity believes that this
-		 * may throw an exception. Actually this code path is not
-		 * taken when buffer_ == 0 */
-		buffer_->invalidateBibfileCache();
 	delete label_;
 }
 
@@ -354,8 +349,6 @@ void InsetInclude::setParams(InsetCommandParams const & p)
 
 	if (type(params()) == INPUT)
 		add_preview(*preview_, *this, buffer());
-
-	buffer().invalidateBibfileCache();
 }
 
 
@@ -769,7 +762,7 @@ void InsetInclude::latex(otexstream & os, OutputParams const & runparams) const
 						"uses module `%2$s'\n"
 						"which is not used in parent file."),
 						included_file.displayName(), from_utf8(module));
-					Alert::warning(_("Module not found"), text);
+					Alert::warning(_("Module not found"), text, true);
 				}
 			}
 		}
