@@ -69,7 +69,7 @@ NoteTranslator const & notetranslator()
 }
 
 
-} // anon
+} // namespace
 
 
 InsetNoteParams::InsetNoteParams()
@@ -100,7 +100,7 @@ void InsetNoteParams::read(Lexer & lex)
 /////////////////////////////////////////////////////////////////////
 
 InsetNote::InsetNote(Buffer * buf, string const & label)
-	: InsetCollapsable(buf)
+	: InsetCollapsible(buf)
 {
 	params_.type = notetranslator().find(label);
 }
@@ -127,14 +127,14 @@ Inset::DisplayType InsetNote::display() const
 void InsetNote::write(ostream & os) const
 {
 	params_.write(os);
-	InsetCollapsable::write(os);
+	InsetCollapsible::write(os);
 }
 
 
 void InsetNote::read(Lexer & lex)
 {
 	params_.read(lex);
-	InsetCollapsable::read(lex);
+	InsetCollapsible::read(lex);
 }
 
 
@@ -176,7 +176,7 @@ void InsetNote::doDispatch(Cursor & cur, FuncRequest & cmd)
 		break;
 
 	default:
-		InsetCollapsable::doDispatch(cur, cmd);
+		InsetCollapsible::doDispatch(cur, cmd);
 		break;
 	}
 }
@@ -203,39 +203,18 @@ bool InsetNote::getStatus(Cursor & cur, FuncRequest const & cmd,
 		return true;
 
 	default:
-		return InsetCollapsable::getStatus(cur, cmd, flag);
+		return InsetCollapsible::getStatus(cur, cmd, flag);
 	}
 }
 
 
-void InsetNote::addToToc(DocIterator const & cpit, bool output_active,
-						 UpdateType utype) const
-{
-	DocIterator pit = cpit;
-	pit.push_back(CursorSlice(const_cast<InsetNote &>(*this)));
-	
-	InsetLayout const & il = getLayout();
-	docstring tooltip;
-	text().forOutliner(tooltip, TOC_ENTRY_LENGTH);
-	docstring str = translateIfPossible(il.labelstring()) + ": " + tooltip;
-	tooltip = support::wrapParas(tooltip, 0, 60, 2);
-	
-	shared_ptr<Toc> toc = buffer().tocBackend().toc("note");
-	toc->push_back(TocItem(pit, 0, str, output_active, tooltip));
-
-	// Proceed with the rest of the inset.
-	bool doing_output = output_active && producesOutput();
-	InsetCollapsable::addToToc(cpit, doing_output, utype);
-}
-
-
 bool InsetNote::isMacroScope() const
 {
 	// LyX note has no latex output
 	if (params_.type == InsetNoteParams::Note)
 		return true;
 
-	return InsetCollapsable::isMacroScope();
+	return InsetCollapsible::isMacroScope();
 }
 
 
@@ -249,7 +228,7 @@ void InsetNote::latex(otexstream & os, OutputParams const & runparams_in) const
 		runparams.inComment = true;
 		// Ignore files that are exported inside a comment
 		runparams.exportdata.reset(new ExportData);
-	} 
+	}
 
 	// the space after the comment in 'a[comment] b' will be eaten by the
 	// comment environment since the space before b is ignored with the
@@ -266,7 +245,7 @@ void InsetNote::latex(otexstream & os, OutputParams const & runparams_in) const
 	// insets whose InsetLayout Display tag is false. This is achieved
 	// by telling otexstream to protect an immediately following space
 	// and is done for both comment and greyedout insets.
-	InsetCollapsable::latex(os, runparams);
+	InsetCollapsible::latex(os, runparams);
 
 	runparams_in.encoding = runparams.encoding;
 }
@@ -321,7 +300,7 @@ docstring InsetNote::xhtml(XHTMLStream & xs, OutputParams const & rp) const
 	if (params_.type == InsetNoteParams::Note)
 		return docstring();
 
-	return InsetCollapsable::xhtml(xs, rp);
+	return InsetCollapsible::xhtml(xs, rp);
 }
 
 
@@ -332,13 +311,13 @@ void InsetNote::validate(LaTeXFeatures & features) const
 		if (features.runparams().flavor == OutputParams::HTML)
 			// we do output this but set display to "none" by default,
 			// but people might want to use it.
-			InsetCollapsable::validate(features);
+			InsetCollapsible::validate(features);
 		else
 			// Only do the requires
 			features.useInsetLayout(getLayout());
 		break;
 	case InsetNoteParams::Greyedout:
-		InsetCollapsable::validate(features);
+		InsetCollapsible::validate(features);
 		break;
 	case InsetNoteParams::Note:
 		break;
