@@ -339,7 +339,9 @@ public:
 		///
 		BOX_PARBOX = 1,
 		///
-		BOX_MINIPAGE = 2
+		BOX_MINIPAGE = 2,
+		///
+		BOX_VARWIDTH = 3
 	};
 
 	enum CaptionType {
@@ -798,20 +800,23 @@ public:
 	///
 	// helper function for Latex
 	///
-	void TeXTopHLine(otexstream &, row_type row, std::string const & lang) const;
+	void TeXTopHLine(otexstream &, row_type row, std::string const & lang,
+			 std::list<col_type>) const;
 	///
-	void TeXBottomHLine(otexstream &, row_type row, std::string const & lang) const;
+	void TeXBottomHLine(otexstream &, row_type row, std::string const & lang,
+			    std::list<col_type>) const;
 	///
-	void TeXCellPreamble(otexstream &, idx_type cell, bool & ismulticol, bool & ismultirow) const;
+	void TeXCellPreamble(otexstream &, idx_type cell, bool & ismulticol, bool & ismultirow,
+			     bool const bidi) const;
 	///
 	void TeXCellPostamble(otexstream &, idx_type cell, bool ismulticol, bool ismultirow) const;
 	///
-	void TeXLongtableHeaderFooter(otexstream &, OutputParams const &) const;
+	void TeXLongtableHeaderFooter(otexstream &, OutputParams const &, std::list<col_type>) const;
 	///
 	bool isValidRow(row_type const row) const;
 	///
 	void TeXRow(otexstream &, row_type const row,
-		    OutputParams const &) const;
+		    OutputParams const &, std::list<col_type>) const;
 	///
 	// helper functions for plain text
 	///
