@@ -31,8 +31,6 @@
 #include "support/lassert.h"
 #include "support/lstrings.h"
 
-#include "support/bind.h"
-
 using namespace std;
 using namespace lyx::support;
 
@@ -77,19 +75,11 @@ RenderPreview::RenderPreview(Inset const * inset)
 RenderPreview::RenderPreview(RenderPreview const & other,
 			     Inset const * inset)
 	: RenderBase(other),
-	  boost::signals::trackable(),
 	  snippet_(other.snippet_),
 	  parent_(inset)
 {}
 
 
-RenderPreview::~RenderPreview()
-{
-	if (ploader_connection_.connected())
-		ploader_connection_.disconnect();
-}
-
-
 RenderBase * RenderPreview::clone(Inset const * inset) const
 {
 	return new RenderPreview(*this, inset);
@@ -104,6 +94,9 @@ docstring const statusMessage(BufferView const * bv, string const & snippet)
 
 	Buffer const & buffer = bv->buffer();
 	graphics::PreviewLoader const * loader = buffer.loader();
+	// please coverity (probably worth the check anyway)
+	if (!loader)
+		return docstring();
 	graphics::PreviewLoader::Status const status = loader->status(snippet);
 
 	docstring message;
@@ -123,7 +116,7 @@ docstring const statusMessage(BufferView const * bv, string const & snippet)
 	return message;
 }
 
-} // namespace anon
+} // namespace
 
 
 graphics::PreviewImage const *
@@ -190,6 +183,8 @@ void RenderPreview::draw(PainterInfo & pi, int x, int y) const
 			     y - theFontMetrics(font).maxAscent() - 4,
 			     stat, font);
 	}
+	pi.change_.paintCue(pi, x, y - dim_.asc,
+	                    x + dim_.width(), y - dim_.asc + dim_.height());
 }
 
 
@@ -205,7 +200,7 @@ void RenderPreview::startLoading(Buffer const & buffer, bool forexport) const
 
 
 void RenderPreview::addPreview(docstring const & latex_snippet,
-                               Buffer const & buffer, 
+                               Buffer const & buffer,
                                bool ignore_lyxrc)
 {
 	if (lyxrc.preview == LyXRC::PREVIEW_OFF && !ignore_lyxrc)
@@ -218,7 +213,7 @@ void RenderPreview::addPreview(docstring const & latex_snippet,
 
 
 void RenderPreview::addPreview(docstring const & latex_snippet,
-                               graphics::PreviewLoader & ploader, 
+                               graphics::PreviewLoader & ploader,
                                bool ignore_lyxrc)
 {
 	if (lyxrc.preview == LyXRC::PREVIEW_OFF && !ignore_lyxrc)
@@ -236,10 +231,12 @@ void RenderPreview::addPreview(docstring const & latex_snippet,
 	// If this is the first time of calling, connect to the
 	// PreviewLoader signal that'll inform us when the preview image
 	// is ready for loading.
-	if (!ploader_connection_.connected()) {
-		ploader_connection_ = ploader.connect(
-			bind(&RenderPreview::imageReady, this, _1));
-	}
+	if (!ploader_connection_.connected())
+		// This is a scoped connection.
+		ploader_connection_ =
+			ploader.connect([this](graphics::PreviewImage const & pi){
+				imageReady(pi);
+			});
 
 	ploader.add(snippet_);
 }
@@ -266,28 +263,58 @@ void RenderPreview::imageReady(graphics::PreviewImage const & pimage)
 
 
 RenderMonitoredPreview::RenderMonitoredPreview(Inset const * inset)
-	: RenderPreview(inset), monitor_(FileName(), 2000)
-{}
+	: RenderPreview(inset)
+{
+	setAbsFile(FileName());
+}
 
 
 void RenderMonitoredPreview::setAbsFile(FileName const & file)
 {
-	monitor_.reset(file);
+	bool mon = monitoring();
+	if (mon)
+		stopMonitoring();
+	filename_ = file;
+	if (mon)
+		startMonitoring();
 }
 
 
 void RenderMonitoredPreview::draw(PainterInfo & pi, int x, int y) const
 {
 	RenderPreview::draw(pi, x, y);
-	if (!monitoring())
-		startMonitoring();
+	startMonitoring();
+	monitor_->checkModifiedAsync();
 }
 
 
-boost::signals::connection
-RenderMonitoredPreview::fileChanged(slot_type const & slot)
+signals2::connection RenderMonitoredPreview::connect(slot const & slot)
 {
-	return monitor_.connect(slot);
+	return changed_.connect(slot);
 }
 
+
+bool RenderMonitoredPreview::monitoring() const
+{
+	return (bool) monitor_;
+}
+
+
+void RenderMonitoredPreview::startMonitoring() const
+{
+	if (!monitoring()) {
+		monitor_ = FileSystemWatcher::activeMonitor(filename_);
+		monitor_->connect([this](bool /* exists */){ changed_(); });
+	}
+}
+
+
+void RenderMonitoredPreview::stopMonitoring() const
+{
+	monitor_ = nullptr;
+}
+
+
+
+
 } // namespace lyx
