@@ -69,21 +69,8 @@ void InsetMathDelim::validate(LaTeXFeatures & features) const
 	// The delimiters may be used without \left or \right as well.
 	// Therefore they are listed in lib/symbols, and if they have
 	// requirements, we need to add them here.
-	MathWordList const & words = mathedWordList();
-	MathWordList::const_iterator it = words.find(left_);
-	if (it != words.end())
-	{
-		string const req = it->second.requires;
-		if (!req.empty())
-			features.require(req);
-	}
-	it = words.find(right_);
-	if (it != words.end())
-	{
-		string const req = it->second.requires;
-		if (!req.empty())
-			features.require(req);
-	}
+	validate_math_word(features, left_);
+	validate_math_word(features, right_);
 }
 
 
@@ -104,6 +91,7 @@ void InsetMathDelim::normalize(NormalStream & os) const
 
 void InsetMathDelim::metrics(MetricsInfo & mi, Dimension & dim) const
 {
+	Changer dummy = mi.base.changeEnsureMath();
 	Dimension dim0;
 	cell(0).metrics(mi, dim0);
 	Dimension t = theFontMetrics(mi.base.font).dimension('I');
@@ -115,7 +103,7 @@ void InsetMathDelim::metrics(MetricsInfo & mi, Dimension & dim) const
 		dw_ = 8;
 	if (dw_ < 4)
 		dw_ = 4;
-	dim.wid = dim0.width() + 2 * dw_ + 8;
+	dim.wid = dim0.width() + 2 * dw_ + 2 * mathed_thinmuskip(mi.base.font);
 	dim.asc = max(a0, d0) + h0;
 	dim.des = max(a0, d0) - h0;
 }
@@ -123,13 +111,14 @@ void InsetMathDelim::metrics(MetricsInfo & mi, Dimension & dim) const
 
 void InsetMathDelim::draw(PainterInfo & pi, int x, int y) const
 {
+	Changer dummy = pi.base.changeEnsureMath();
 	Dimension const dim = dimension(*pi.base.bv);
 	int const b = y - dim.asc;
-	cell(0).draw(pi, x + dw_ + 4, y);
-	mathed_draw_deco(pi, x + 4, b, dw_, dim.height(), left_);
-	mathed_draw_deco(pi, x + dim.width() - dw_ - 4,
+	int const skip = mathed_thinmuskip(pi.base.font);
+	cell(0).draw(pi, x + dw_ + skip, y);
+	mathed_draw_deco(pi, x + skip / 2, b, dw_, dim.height(), left_);
+	mathed_draw_deco(pi, x + dim.width() - dw_ - skip / 2,
 		b, dw_, dim.height(), right_);
-	setPosCache(pi, x, y);
 }
 
 
@@ -193,19 +182,19 @@ void InsetMathDelim::mathematica(MathematicaStream & os) const
 void InsetMathDelim::mathmlize(MathStream & os) const
 {
 	os << "<mo form='prefix' fence='true' stretchy='true' symmetric='true'>"
-	   << convertDelimToXMLEscape(left_) 
+	   << convertDelimToXMLEscape(left_)
 	   << "</mo>\n"
-	   << cell(0) 
-	   << "\n<mo form='postfix' fence='true' stretchy='true' symmetric='true'>" 
-	   << convertDelimToXMLEscape(right_) 
+	   << cell(0)
+	   << "\n<mo form='postfix' fence='true' stretchy='true' symmetric='true'>"
+	   << convertDelimToXMLEscape(right_)
 	   << "</mo>\n";
 }
 
 
 void InsetMathDelim::htmlize(HtmlStream & os) const
 {
-	os << convertDelimToXMLEscape(left_) 
-	   << cell(0) 
+	os << convertDelimToXMLEscape(left_)
+	   << cell(0)
 	   << convertDelimToXMLEscape(right_);
 }
 
