@@ -19,6 +19,7 @@
 #include "MetricsInfo.h"
 
 #include "support/gettext.h"
+#include "support/lassert.h"
 #include "support/lstrings.h"
 
 #include <ostream>
@@ -38,6 +39,13 @@ Inset * InsetMathFont::clone() const
 }
 
 
+std::string InsetMathFont::font() const
+{
+	LASSERT(isAscii(key_->name), return "mathnormal");
+	return to_ascii(key_->name);
+}
+
+
 InsetMath::mode_type InsetMathFont::currentMode() const
 {
 	if (key_->extra == "mathmode")
@@ -56,20 +64,35 @@ bool InsetMathFont::lockedMode() const
 }
 
 
+void InsetMathFont::write(WriteStream & os) const
+{
+	// Close the mode changing command inserted during export if
+	// we are going to output another mode changing command that
+	// actually doesn't change mode. This avoids exporting things
+	// such as \ensuremath{a\mathit{b}} or \textit{a\text{b}} and
+	// produce instead \ensuremath{a}\mathit{b} and \textit{a}\text{b}.
+	if (os.pendingBrace()
+	    && ((currentMode() == TEXT_MODE && os.textMode())
+		    || (currentMode() == MATH_MODE && !os.textMode()))) {
+		os.os() << '}';
+		os.pendingBrace(false);
+		os.textMode(!os.textMode());
+	}
+	InsetMathNest::write(os);
+}
+
+
 void InsetMathFont::metrics(MetricsInfo & mi, Dimension & dim) const
 {
-	FontSetChanger dummy(mi.base, key_->name);
+	Changer dummy = mi.base.changeFontSet(font());
 	cell(0).metrics(mi, dim);
-	metricsMarkers(dim);
 }
 
 
 void InsetMathFont::draw(PainterInfo & pi, int x, int y) const
 {
-	FontSetChanger dummy(pi.base, key_->name);
-	cell(0).draw(pi, x + 1, y);
-	drawMarkers(pi, x, y);
-	setPosCache(pi, x, y);
+	Changer dummy = pi.base.changeFontSet(font());
+	cell(0).draw(pi, x, y);
 }
 
 
@@ -96,19 +119,20 @@ docstring InsetMathFont::name() const
 void InsetMathFont::validate(LaTeXFeatures & features) const
 {
 	InsetMathNest::validate(features);
+	std::string fontname = font();
 	if (features.runparams().isLaTeX()) {
 		// Make sure amssymb is put in preamble if Blackboard Bold or
 		// Fraktur used:
-		if (key_->name == "mathfrak" || key_->name == "mathbb")
+		if (fontname == "mathfrak" || fontname == "mathbb")
 			features.require("amssymb");
-		if (key_->name == "text" || key_->name == "textnormal"
-				|| (key_->name.length() == 6 && key_->name.substr(0, 4) == "text"))
+		if (fontname == "text" || fontname == "textnormal"
+		    || (fontname.length() == 6 && fontname.substr(0, 4) == "text"))
 			features.require("amstext");
-		if (key_->name == "mathscr")
+		if (fontname == "mathscr")
 			features.require("mathrsfs");
-		if (key_->name == "textipa")
+		if (fontname == "textipa")
 			features.require("tipa");
-		if (key_->name == "ce" || key_->name == "cf")
+		if (fontname == "ce" || fontname == "cf")
 			features.require("mhchem");
 	} else if (features.runparams().math_flavor == OutputParams::MathAsHTML) {
 		features.addCSSSnippet(
