@@ -44,13 +44,6 @@ public:
 		Multicolumn multi_;
 		/// special multi colums alignment
 		docstring align_;
-		/// these should be a per-cell property, but ok to have it here
-		/// for single-column grids like paragraphs
-		mutable int glue_;
-		///
-		mutable pos_type begin_;
-		///
-		mutable pos_type end_;
 	};
 
 	/// additional per-row information
@@ -107,16 +100,12 @@ public:
 	InsetMathGrid(Buffer * buf, col_type m, row_type n, char valign,
 		docstring const & halign);
 	///
+	marker_type marker(BufferView const *) const { return MARKER2; };
+	///
 	void metrics(MetricsInfo & mi, Dimension &) const;
 	///
 	void draw(PainterInfo & pi, int x, int y) const;
 	///
-	void drawWithMargin(PainterInfo & pi, int x, int y,
-		int lmargin = 0, int rmargin = 0) const;
-	/// draw decorations.
-	void drawDecoration(PainterInfo & pi, int x, int y) const
-	{ drawMarkers2(pi, x, y); }
-	///
 	void metricsT(TextMetricsInfo const & mi, Dimension & dim) const;
 	///
 	void drawT(TextPainter & pi, int x, int y) const;
@@ -221,7 +210,7 @@ public:
 	virtual int vlinesep() const;
 	///
 	virtual int border() const;
-	/// 
+	///
 	virtual bool handlesMulticolumn() const { return false; }
 
 	///
@@ -236,7 +225,7 @@ public:
 	//void maple(MapleStream &) const;
 	///
 	void mathmlize(MathStream &) const;
-	/// 
+	///
 	void htmlize(HtmlStream &) const;
 	///
 	void htmlize(HtmlStream &, std::string attrib) const;
@@ -257,6 +246,11 @@ protected:
 	int cellYOffset(idx_type idx) const;
 	/// Width of cell, taking combined columns into account
 	int cellWidth(idx_type idx) const;
+	///
+	virtual int leftMargin() const { return 0; }
+	///
+	virtual int rightMargin() const { return 0; }
+
 	/// returns proper 'end of line' code for LaTeX
 	virtual docstring eolString(row_type row, bool fragile, bool latex,
 			bool last_eoln) const;
@@ -273,11 +267,15 @@ protected:
 
 	// The following two functions are used in InsetMathHull and
 	// InsetMathSplit.
-	/// The value of a fixed col align for a certain hull type 
+	/// The value of a fixed col align for a certain hull type
 	static char colAlign(HullType type, col_type col);
 	/// The value of a fixed col spacing for a certain hull type
 	static int colSpace(HullType type, col_type col);
 
+	/// positions of vertical and horizontal lines
+	int vLineHOffset(col_type col, unsigned int line) const;
+	int hLineVOffset(row_type row, unsigned int line) const;
+
 	/// row info.
 	/// rowinfo_[nrows()] is a dummy row used only for hlines.
 	std::vector<RowInfo> rowinfo_;
@@ -291,7 +289,7 @@ protected:
 
 private:
 	///
-	char v_align_; // add approp. type
+	char v_align_; // FIXME: add approp. type
 	///
 	Inset * clone() const;
 };
