@@ -17,7 +17,7 @@
 #include "DocIterator.h"
 #include "OutputEnums.h"
 
-#include <boost/scoped_ptr.hpp>
+#include "support/unique_ptr.h"
 
 
 namespace lyx {
@@ -52,14 +52,14 @@ public:
 	void updateBuffer(ParIterator const &, UpdateType);
 	///
 	void addToToc(DocIterator const & di, bool output_active,
-				  UpdateType utype) const;
+				  UpdateType utype, TocBackend & backend) const;
 	///
 	InsetMathHull & operator=(InsetMathHull const &);
 	///
 	mode_type currentMode() const;
 	///
 	void metrics(MetricsInfo & mi, Dimension & dim) const;
-	/// 
+	///
 	void drawBackground(PainterInfo & pi, int x, int y) const;
 	///
 	void draw(PainterInfo &, int x, int y) const;
@@ -143,7 +143,7 @@ public:
 	///
 	bool readQuiet(Lexer & lex);
 	///
-	int plaintext(odocstringstream &, OutputParams const &, 
+	int plaintext(odocstringstream &, OutputParams const &,
 	              size_t max_length = INT_MAX) const;
 	///
 	int docbook(odocstream &, OutputParams const &) const;
@@ -155,7 +155,7 @@ public:
 	void htmlize(HtmlStream &) const;
 	///
 	void mathAsLatex(WriteStream &) const;
-	/// 
+	///
 	void toString(odocstream &) const;
 	///
 	void forOutliner(docstring &, size_t const, bool const) const;
@@ -177,7 +177,7 @@ public:
 
 	///
 	static int displayMargin() { return 12; }
-	
+
 	/// Force inset into LTR environment if surroundings are RTL
 	virtual bool forceLTR() const { return true; }
 	///
@@ -187,6 +187,10 @@ public:
 	std::string contextMenuName() const;
 	///
 	InsetCode lyxCode() const { return MATH_HULL_CODE; }
+	///
+	bool canPaintChange(BufferView const &) const;
+	///
+	bool confirmDeletion() const { return nargs() != 1 || !cell(0).empty(); }
 
 protected:
 	InsetMathHull(InsetMathHull const &);
@@ -196,6 +200,12 @@ protected:
 	/// do we want to handle this event?
 	bool getStatus(Cursor & cur, FuncRequest const & cmd,
 		FuncStatus & status) const;
+	/// override to set to 0 for inline equation
+	int leftMargin() const;
+	/// override to set to 0 for inline equation
+	int rightMargin() const;
+	/// override to set to 0 for inline equation
+	int border() const;
 	///
 	docstring eolString(row_type row, bool fragile, bool latex,
 			bool last_eoln) const;
@@ -207,7 +217,7 @@ private:
 	/// If so, we ignore LyXRC and wait for the image to be generated.
 	void preparePreview(DocIterator const & pos,
 	                    bool forexport = false) const;
-	/// like reloadPreview, but forces load 
+	/// like reloadPreview, but forces load
 	/// used by image export
 	void loadPreview(DocIterator const & pos) const;
 	///
@@ -237,7 +247,7 @@ private:
 	/// change number of columns, split or combine columns if necessary.
 	void changeCols(col_type);
 	///
-	docstring standardFont() const;
+	std::string standardFont() const;
 	///
 	ColorCode standardColor() const;
 	/// consistency check
@@ -250,6 +260,8 @@ private:
 	bool colChangeOK() const;
 	/// are any of the equations numbered?
 	bool haveNumbers() const;
+	/// draw four angular markers
+	virtual void drawMarkers(PainterInfo & pi, int x, int y) const;
 
 	/// "none", "simple", "display", "eqnarray",...
 	HullType type_;
@@ -260,7 +272,7 @@ private:
 	///
 	std::vector<InsetLabel *> label_;
 	///
-	boost::scoped_ptr<RenderPreview> preview_;
+	unique_ptr<RenderPreview> preview_;
 	///
 	DocIterator docit_;
 	///
@@ -278,12 +290,14 @@ public:
 	///
 	bool editable() const { return true; }
 	///
-	void edit(Cursor & cur, bool front, 
+	void edit(Cursor & cur, bool front,
 		EntryDirection entry_from = ENTRY_DIRECTION_IGNORE);
 	///
 	Inset * editXY(Cursor & cur, int x, int y);
 	///
 	DisplayType display() const;
+	///
+	int indent(BufferView const &) const;
 
 protected:
 	///
