@@ -11,10 +11,12 @@
 #include <config.h>
 
 #include "InsetMathKern.h"
-#include "MathStream.h"
+
 #include "MathStream.h"
 #include "MathSupport.h"
+
 #include "Dimension.h"
+#include "MetricsInfo.h"
 
 #include "support/docstring.h"
 
@@ -59,6 +61,8 @@ void InsetMathKern::write(WriteStream & os) const
 {
 	if (wid_.empty())
 		os << "\\kern" << ' ';
+	else if (wid_.unit() == Length::MU)
+		os << "\\mkern" << from_utf8(wid_.asLatexString()) << ' ';
 	else
 		os << "\\kern" << from_utf8(wid_.asLatexString()) << ' ';
 }
@@ -73,4 +77,12 @@ void InsetMathKern::normalize(NormalStream & os) const
 }
 
 
+void InsetMathKern::infoize2(odocstream & os) const
+{
+	os << "Kern";
+	if (!wid_.empty())
+		os << ": " << from_utf8(wid_.asLatexString());
+}
+
+
 } // namespace lyx
