@@ -231,25 +231,17 @@ void InsetDisplayLabelBox::metrics(MetricsInfo & mi, Dimension & dim) const
 {
 	InsetLabelBox::metrics(mi, dim);
 	if (!parent_.editing(mi.base.bv)
-	    && parent_.cell(parent_.displayIdx()).empty()) {
-		dim.wid = 0;
-		dim.asc = 0;
-		dim.des = 0;
-	}
+	    && parent_.cell(parent_.displayIdx()).empty())
+		dim.clear();
 }
 
 
 void InsetDisplayLabelBox::draw(PainterInfo & pi, int x, int y) const
 {
 	if (parent_.editing(pi.base.bv)
-	    || !parent_.cell(parent_.displayIdx()).empty()) {
-		InsetLabelBox::draw(pi, x, y);
-	} else {
-		bool enabled = pi.pain.isDrawingEnabled();
-		pi.pain.setDrawingEnabled(false);
-		InsetLabelBox::draw(pi, x, y);
-		pi.pain.setDrawingEnabled(enabled);
-	}
+	    || !parent_.cell(parent_.displayIdx()).empty()
+		|| pi.pain.isNull())
+	    InsetLabelBox::draw(pi, x, y);
 }
 
 
@@ -1225,7 +1217,7 @@ int InsetMathMacroTemplate::write(WriteStream & os, bool overwriteRedefinition)
 			//   \global\def\long\foo#1#2{#1,#2}
 			// We use the \long prefix as this is the equivalent to \newcommand.
 			// We cannot use \newcommand directly because \global does not work with it.
-			os << "\\global\\long\\def\\" << name();
+			os << "\n\\global\\long\\def\\" << name();
 			docstring param = from_ascii("#0");
 			for (int i = 1; i <= numargs_; ++i) {
 				param[1] = '0' + i;
@@ -1257,7 +1249,7 @@ int InsetMathMacroTemplate::write(WriteStream & os, bool overwriteRedefinition)
 
 	if (os.latex()) {
 		// writing .tex. done.
-		os << "\n";
+		os << "%\n";
 		++num_lines;
 	} else {
 		// writing .lyx, write special .tex export only if necessary
