@@ -12,19 +12,20 @@
 
 #include "InsetMathSymbol.h"
 
-#include "Dimension.h"
-#include "LaTeXFeatures.h"
 #include "MathAtom.h"
 #include "MathParser.h"
 #include "MathStream.h"
 #include "MathSupport.h"
 
+#include "Dimension.h"
+#include "LaTeXFeatures.h"
+#include "MetricsInfo.h"
+
 #include "support/debug.h"
 #include "support/docstream.h"
 #include "support/lyxlib.h"
 #include "support/textutils.h"
-
-#include <boost/scoped_ptr.hpp>
+#include "support/unique_ptr.h"
 
 
 namespace lyx {
@@ -59,34 +60,19 @@ docstring InsetMathSymbol::name() const
 
 void InsetMathSymbol::metrics(MetricsInfo & mi, Dimension & dim) const
 {
-	//lyxerr << "metrics: symbol: '" << sym_->name
-	//	<< "' in font: '" << sym_->inset
-	//	<< "' drawn as: '" << sym_->draw
-	//	<< "'" << endl;
-
-	bool const italic_upcase_greek = sym_->inset == "cmr" &&
-					 sym_->extra == "mathalpha" &&
-					 mi.base.fontname == "mathit";
-	std::string const font = italic_upcase_greek ? "cmm" : sym_->inset;
-	int const em = mathed_font_em(mi.base.font);
-	FontSetChanger dummy(mi.base, from_ascii(font));
-	mathed_string_dim(mi.base.font, sym_->draw, dim);
-	docstring::const_reverse_iterator rit = sym_->draw.rbegin();
-	kerning_ = mathed_char_kerning(mi.base.font, *rit);
+	// set dim
+	mathedSymbolDim(mi.base, dim, sym_);
+	// set kerning_
+	kerning_ = mathed_char_kerning(mi.base.font, *sym_->draw.rbegin());
 	// correct height for broken cmex and wasy font
 	if (sym_->inset == "cmex" || sym_->inset == "wasy") {
 		h_ = 4 * dim.des / 5;
 		dim.asc += h_;
 		dim.des -= h_;
 	}
-	// seperate things a bit
-	if (isRelOp())
-		dim.wid += support::iround(0.5 * em);
-	else
-		dim.wid += support::iround(0.1667 * em);
-
+	// set scriptable_
 	scriptable_ = false;
-	if (mi.base.style == LM_ST_DISPLAY)
+	if (mi.base.font.style() == LM_ST_DISPLAY)
 		if (sym_->inset == "cmex" || sym_->inset == "esint" ||
 		    sym_->extra == "funclim" ||
 		    (sym_->inset == "stmry" && sym_->extra == "mathop"))
@@ -96,23 +82,7 @@ void InsetMathSymbol::metrics(MetricsInfo & mi, Dimension & dim) const
 
 void InsetMathSymbol::draw(PainterInfo & pi, int x, int y) const
 {
-	//lyxerr << "metrics: symbol: '" << sym_->name
-	//	<< "' in font: '" << sym_->inset
-	//	<< "' drawn as: '" << sym_->draw
-	//	<< "'" << endl;
-
-	bool const italic_upcase_greek = sym_->inset == "cmr" &&
-					 sym_->extra == "mathalpha" &&
-					 pi.base.fontname == "mathit";
-	std::string const font = italic_upcase_greek ? "cmm" : sym_->inset;
-	int const em = mathed_font_em(pi.base.font);
-	if (isRelOp())
-		x += support::iround(0.25 * em);
-	else
-		x += support::iround(0.0833 * em);
-
-	FontSetChanger dummy(pi.base, from_ascii(font));
-	pi.draw(x, y - h_, sym_->draw);
+	mathedSymbolDraw(pi, x, y - h_, sym_);
 }
 
 
@@ -122,18 +92,21 @@ InsetMath::mode_type InsetMathSymbol::currentMode() const
 }
 
 
-bool InsetMathSymbol::isRelOp() const
-{
-	return sym_->extra == "mathrel";
-}
-
-
 bool InsetMathSymbol::isOrdAlpha() const
 {
 	return sym_->extra == "mathord" || sym_->extra == "mathalpha";
 }
 
 
+MathClass InsetMathSymbol::mathClass() const
+{
+	if (sym_->extra == "func" || sym_->extra == "funclim")
+		return MC_OP;
+	MathClass const mc = string_to_class(sym_->extra);
+	return (mc == MC_UNKNOWN) ? MC_ORD : mc;
+}
+
+
 bool InsetMathSymbol::isScriptable() const
 {
 	return scriptable_;
@@ -238,9 +211,11 @@ void InsetMathSymbol::octave(OctaveStream & os) const
 
 void InsetMathSymbol::write(WriteStream & os) const
 {
-	boost::scoped_ptr<MathEnsurer> ensurer;
+	unique_ptr<MathEnsurer> ensurer;
 	if (currentMode() != TEXT_MODE)
-		ensurer.reset(new MathEnsurer(os));
+		ensurer = make_unique<MathEnsurer>(os);
+	else
+		ensurer = make_unique<MathEnsurer>(os, false, true, true);
 	os << '\\' << name();
 
 	// $,#, etc. In theory the restriction based on catcodes, but then
