@@ -264,25 +264,50 @@ bool isInside(DocIterator const & it, MathData const & ar,
 #endif
 
 
-void MathData::metrics(MetricsInfo & mi, Dimension & dim) const
+bool MathData::hasCaret(BufferView * bv) const
+{
+	Cursor & cur = bv->cursor();
+	return cur.inMathed() && &cur.cell() == this;
+}
+
+
+void MathData::metrics(MetricsInfo & mi, Dimension & dim, bool tight) const
 {
 	frontend::FontMetrics const & fm = theFontMetrics(mi.base.font);
-	dim = fm.dimension('I');
+	BufferView * bv = mi.base.bv;
+	int const Iascent = fm.dimension('I').ascent();
 	int xascent = fm.dimension('x').ascent();
-	if (xascent >= dim.asc)
-		xascent = (2 * dim.asc) / 3;
+	if (xascent >= Iascent)
+		xascent = (2 * Iascent) / 3;
 	minasc_ = xascent;
 	mindes_ = (3 * xascent) / 4;
 	slevel_ = (4 * xascent) / 5;
 	sshift_ = xascent / 4;
 
 	MathRow mrow(mi, this);
-	mrow.metrics(mi, dim);
-	mrow_cache_[mi.base.bv] = mrow;
-	kerning_ = mrow.kerning(mi.base.bv);
+	bool has_caret = mrow.metrics(mi, dim);
+	mrow_cache_[bv] = mrow;
+	kerning_ = mrow.kerning(bv);
+
+	// Set a minimal ascent/descent for the cell
+	if (tight)
+		// FIXME: this is the minimal ascent seen empirically, check
+		// what the TeXbook says.
+		dim.asc = max(dim.asc, fm.ascent('x'));
+	else {
+		dim.asc = max(dim.asc, fm.maxAscent());
+		dim.des = max(dim.des, fm.maxDescent());
+	}
+
+	// This is one of the the few points where the drawing font is known,
+	// so that we can set the caret vertical dimensions.
+	has_caret |= hasCaret(bv);
+	if (has_caret)
+		bv->setCaretAscentDescent(min(dim.asc, fm.maxAscent()),
+		                          min(dim.des, fm.maxDescent()));
 
 	// Cache the dimension.
-	mi.base.bv->coordCache().arrays().add(this, dim);
+	bv->coordCache().arrays().add(this, dim);
 }
 
 
