@@ -19,6 +19,7 @@
 #include "InsetMathCancel.h"
 #include "InsetMathCancelto.h"
 #include "InsetMathCases.h"
+#include "InsetMathClass.h"
 #include "InsetMathColor.h"
 #include "InsetMathDecoration.h"
 #include "InsetMathDots.h"
@@ -49,8 +50,8 @@
 #include "InsetMathXYMatrix.h"
 #include "InsetMathDiagram.h"
 #include "MacroTable.h"
-#include "MathMacro.h"
-#include "MathMacroArgument.h"
+#include "InsetMathMacro.h"
+#include "InsetMathMacroArgument.h"
 #include "MathParser.h"
 #include "MathStream.h"
 #include "MathSupport.h"
@@ -73,6 +74,8 @@
 #include "LyX.h" // use_gui
 #include "OutputParams.h"
 
+#include <iomanip>
+
 using namespace std;
 using namespace lyx::support;
 
@@ -92,7 +95,7 @@ bool isMathFontAvailable(string & name)
 		return false;
 
 	FontInfo f;
-	augmentFont(f, from_ascii(name));
+	augmentFont(f, name);
 
 	// Do we have the font proper?
 	if (theFontLoader().available(f))
@@ -146,6 +149,8 @@ void initSymbols()
 	}
 
 	ifstream fs(filename.toFilesystemEncoding().c_str());
+	// limit the size of strings we read to avoid memory problems
+	fs >> setw(65636);
 	string line;
 	bool skip = false;
 	while (getline(fs, line)) {
@@ -157,6 +162,8 @@ void initSymbols()
 		// special case of iffont/else/endif
 		if (line.size() >= 7 && line.substr(0, 6) == "iffont") {
 			istringstream is(line);
+			// limit the size of strings we read to avoid memory problems
+			is >> setw(65636);
 			string tmp;
 			is >> tmp;
 			is >> tmp;
@@ -186,7 +193,7 @@ void initSymbols()
 			string extra;
 			string xmlname;
 			bool hidden = false;
-			is >> macro >> requires;
+			is >> setw(65536) >> macro >> requires;
 			if ((is >> xmlname)) {
 				extra = requires;
 				if (!(is >> requires))
@@ -233,7 +240,7 @@ void initSymbols()
 		docstring help;
 		is >> tmp.name >> help;
 		tmp.inset = to_ascii(help);
-		if (isFontName(help))
+		if (isFontName(tmp.inset))
 			is >> charid >> fallbackid >> tmp.extra >> tmp.xmlname;
 		else
 			is >> tmp.extra;
@@ -253,7 +260,7 @@ void initSymbols()
 			continue;
 		}
 
-		if (isFontName(from_ascii(tmp.inset))) {
+		if (isFontName(tmp.inset)) {
 			// tmp.inset _is_ the fontname here.
 			// create fallbacks if necessary
 
@@ -340,7 +347,7 @@ bool isSpecialChar(docstring const & name)
 }
 
 
-} // namespace anon
+} // namespace
 
 MathWordList const & mathedWordList()
 {
@@ -359,23 +366,41 @@ void initMath()
 }
 
 
-bool ensureMath(WriteStream & os, bool needs_math_mode, bool macro)
+bool ensureMath(WriteStream & os, bool needs_mathmode, bool macro,
+                bool textmode_macro)
 {
 	bool brace = os.pendingBrace();
 	os.pendingBrace(false);
 	if (!os.latex())
 		return brace;
-	if (os.textMode() && needs_math_mode) {
-		os << "\\ensuremath{";
+	if (os.textMode() && needs_mathmode) {
+		if (brace) {
+			// close \lyxmathsym
+			os << '}';
+			brace = false;
+		} else {
+			os << "\\ensuremath{";
+			brace = true;
+		}
 		os.textMode(false);
-		brace = true;
-	} else if (macro && brace && !needs_math_mode) {
-		// This is a user defined macro, but not a MathMacro, so we
-		// cannot be sure what mode is needed. As it was entered in
-		// a text box, we restore the text mode.
-		os << '}';
+	} else if (macro && textmode_macro && !os.textMode()) {
+		if (brace) {
+			// close \ensuremath
+			os << '}';
+			brace = false;
+		} else {
+			os << "\\lyxmathsym{";
+			brace = true;
+		}
 		os.textMode(true);
+	} else if (macro && brace && !needs_mathmode && !textmode_macro) {
+		// This is a user defined macro, not a InsetMathMacro, so we
+		// cannot be sure what mode is needed. We leave it in the
+		// same environment it was entered by closing either \lyxmathsym
+		// or \ensuremath, whichever was opened.
+		os << '}';
 		brace = false;
+		os.textMode(!os.textMode());
 	}
 	return brace;
 }
@@ -423,7 +448,7 @@ MathAtom createInsetMath(docstring const & s, Buffer * buf)
 	//lyxerr << "creating inset with name: '" << to_utf8(s) << '\'' << endl;
 	if ((s == "ce" || s == "cf") && buf
 	    && buf->params().use_package("mhchem") == BufferParams::package_off)
-		return MathAtom(new MathMacro(buf, s));
+		return MathAtom(new InsetMathMacro(buf, s));
 
 	latexkeys const * l = in_word_set(s);
 	if (l) {
@@ -439,6 +464,8 @@ MathAtom createInsetMath(docstring const & s, Buffer * buf)
 			return MathAtom(new InsetMathDecoration(buf, l));
 		if (inset == "space")
 			return MathAtom(new InsetMathSpace(to_ascii(l->name), ""));
+		if (inset == "class")
+			return MathAtom(new InsetMathClass(buf, string_to_class(s)));
 		if (inset == "dots")
 			return MathAtom(new InsetMathDots(l));
 		if (inset == "mbox")
@@ -463,10 +490,10 @@ MathAtom createInsetMath(docstring const & s, Buffer * buf)
 	}
 
 	if (s.size() == 2 && s[0] == '#' && s[1] >= '1' && s[1] <= '9')
-		return MathAtom(new MathMacroArgument(s[1] - '0'));
+		return MathAtom(new InsetMathMacroArgument(s[1] - '0'));
 	if (s.size() == 3 && s[0] == '\\' && s[1] == '#'
 			&& s[2] >= '1' && s[2] <= '9')
-		return MathAtom(new MathMacroArgument(s[2] - '0'));
+		return MathAtom(new InsetMathMacroArgument(s[2] - '0'));
 	if (s == "boxed")
 		return MathAtom(new InsetMathBoxed(buf));
 	if (s == "fbox")
@@ -475,8 +502,6 @@ MathAtom createInsetMath(docstring const & s, Buffer * buf)
 		return MathAtom(new InsetMathMakebox(buf, true));
 	if (s == "makebox")
 		return MathAtom(new InsetMathMakebox(buf, false));
-	if (s == "kern")
-		return MathAtom(new InsetMathKern);
 	if (s.substr(0, 8) == "xymatrix") {
 		char spacing_code = '\0';
 		Length spacing;
@@ -643,11 +668,10 @@ MathAtom createInsetMath(docstring const & s, Buffer * buf)
 		return MathAtom(new InsetMathSpecialChar(s));
 	if (s == " ")
 		return MathAtom(new InsetMathSpace(" ", ""));
-
 	if (s == "regexp")
 		return MathAtom(new InsetMathHull(buf, hullRegexp));
 
-	return MathAtom(new MathMacro(buf, s));
+	return MathAtom(new InsetMathMacro(buf, s));
 }
 
 
@@ -669,9 +693,8 @@ bool createInsetMath_fromDialogStr(docstring const & str, MathData & ar)
 		InsetSpaceParams isp(true);
 		InsetSpace::string2params(to_utf8(str), isp);
 		InsetSpace is(isp);
-		TexRow texrow;
 		odocstringstream ods;
-		otexstream os(ods, texrow);
+		otexstream os(ods);
 		Encoding const * const ascii = encodings.fromLyXName("ascii");
 		OutputParams op(ascii);
 		is.latex(os, op);
