@@ -51,7 +51,7 @@ following hack as starting point to write some macros:
 #include "InsetMathEnv.h"
 #include "InsetMathFrac.h"
 #include "InsetMathKern.h"
-#include "MathMacro.h"
+#include "InsetMathMacro.h"
 #include "InsetMathPar.h"
 #include "InsetMathRef.h"
 #include "InsetMathRoot.h"
@@ -63,10 +63,10 @@ following hack as starting point to write some macros:
 #include "InsetMathStackrel.h"
 #include "InsetMathString.h"
 #include "InsetMathTabular.h"
-#include "MathMacroTemplate.h"
+#include "InsetMathMacroTemplate.h"
 #include "MathExtern.h"
 #include "MathFactory.h"
-#include "MathMacroArgument.h"
+#include "InsetMathMacroArgument.h"
 #include "MathSupport.h"
 
 #include "Buffer.h"
@@ -74,9 +74,10 @@ following hack as starting point to write some macros:
 #include "Encoding.h"
 #include "Lexer.h"
 
-#include "support/debug.h"
 #include "support/convert.h"
+#include "support/debug.h"
 #include "support/docstream.h"
+#include "support/unique_ptr.h"
 
 #include <sstream>
 
@@ -928,8 +929,13 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 		}
 
 		else if (t.cat() == catParameter) {
-			Token const & n	= getToken();
-			cell->push_back(MathAtom(new MathMacroArgument(n.character()-'0')));
+			Token const & n	= nextToken();
+			char_type c = n.character();
+			if (c && '0' < c && c <= '9') {
+				cell->push_back(MathAtom(new InsetMathMacroArgument(c - '0')));
+				getToken();
+			} else
+				cell->push_back(MathAtom(new InsetMathHash()));
 		}
 
 		else if (t.cat() == catActive)
@@ -1076,7 +1082,7 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 			if (nextToken().cat() == catBegin)
 				parse(display, FLAG_ITEM, InsetMath::MATH_MODE);
 
-			cell->push_back(MathAtom(new MathMacroTemplate(buf,
+			cell->push_back(MathAtom(new InsetMathMacroTemplate(buf,
 				name, nargs, 0, MacroTypeDef,
 				vector<MathData>(), def, display)));
 
@@ -1124,7 +1130,7 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 			if (nextToken().cat() == catBegin)
 				parse(display, FLAG_ITEM, InsetMath::MATH_MODE);
 
-			cell->push_back(MathAtom(new MathMacroTemplate(buf,
+			cell->push_back(MathAtom(new InsetMathMacroTemplate(buf,
 				name, nargs, optionals, MacroTypeNewcommand,
 				optionalValues, def, display)));
 
@@ -1245,7 +1251,7 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 			if (nextToken().cat() == catBegin)
 				parse(display, FLAG_ITEM, InsetMath::MATH_MODE);
 
-			cell->push_back(MathAtom(new MathMacroTemplate(buf,
+			cell->push_back(MathAtom(new InsetMathMacroTemplate(buf,
 				name, nargs, optionals, MacroTypeNewcommandx,
 				optionalValues, def, display)));
 
@@ -1368,29 +1374,32 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 			MathData count;
 			parse(count, FLAG_ITEM, mode);
 			int cols;
-			if (extractNumber(count, cols)) {
+			// limit arbitrarily to 100 columns
+			if (extractNumber(count, cols) && cols > 0 && cols < 100) {
 				// resize the table if necessary
 				size_t first = grid.index(cellrow, cellcol);
 				for (int i = 1; i < cols; ++i) {
 					if (addCol(grid, cellcol)) {
 						size_t const idx = grid.index(cellrow, cellcol);
-						grid.cellinfo(idx).multi_ = 
+						grid.cellinfo(idx).multi_ =
 							InsetMathGrid::CELL_PART_OF_MULTICOLUMN;
 					}
 				}
 
-				// the first cell is the real thing, not a dummy 
-				cell = &grid.cell(first); 
+				// the first cell is the real thing, not a dummy
+				cell = &grid.cell(first);
 				grid.cellinfo(first).multi_ =
-					InsetMathGrid::CELL_BEGIN_OF_MULTICOLUMN; 
-				// read special alignment 
-				MathData align; 
-				parse(align, FLAG_ITEM, mode); 
-				grid.cellinfo(first).align_ = asString(align); 
+					InsetMathGrid::CELL_BEGIN_OF_MULTICOLUMN;
+
+				// read special alignment
+				MathData align;
+				parse(align, FLAG_ITEM, mode);
+				grid.cellinfo(first).align_ = asString(align);
+
 				// parse the remaining contents into the "real" cell
-				parse(*cell, FLAG_ITEM, mode); 
+				parse(*cell, FLAG_ITEM, mode);
 			} else {
-				MathAtom at = MathAtom(new MathMacro(buf, t.cs()));
+				MathAtom at = MathAtom(new InsetMathMacro(buf, t.cs()));
 				cell->push_back(at);
 				cell->push_back(MathAtom(new InsetMathBrace(count)));
 			}
@@ -1748,7 +1757,8 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 
 			else {
 				success_ = false;
-				if (!(mode_ & Parse::QUIET)) {
+				if (!(mode_ & Parse::QUIET) &&
+				    !(mode_ & Parse::TRACKMACRO)) {
 					dump();
 					lyxerr << "found unknown math environment '"
 					       << to_utf8(name) << "'" << endl;
@@ -1759,7 +1769,7 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 			}
 		}
 
-		else if (t.cs() == "kern") {
+		else if (t.cs() == "kern" || t.cs() == "mkern") {
 			// FIXME: A hack...
 			docstring s;
 			int num_tokens = 0;
@@ -1777,7 +1787,7 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 					break;
 			}
 			if (s.empty())
-				cell->push_back(MathAtom(new InsetMathKern));
+				cell->push_back(MathAtom(new InsetMathMacro(buf, t.cs())));
 			else
 				cell->push_back(MathAtom(new InsetMathKern(s)));
 		}
@@ -1887,7 +1897,7 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 			else {
 				// Since the Length class cannot use length variables
 				// we must not create an InsetMathSpace.
-				cell->push_back(MathAtom(new MathMacro(buf, name)));
+				cell->push_back(MathAtom(new InsetMathMacro(buf, name)));
 				MathData ar;
 				mathed_parse_cell(ar, '{' + arg + '}', mode_);
 				cell->append(ar);
@@ -1906,7 +1916,7 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 					parse(cell->back().nucleus()->cell(0), FLAG_ITEM, mode);
 				} else {
 					docstring const arg = parse_verbatim_item();
-					cell->push_back(MathAtom(new MathMacro(buf, t.cs())));
+					cell->push_back(MathAtom(new InsetMathMacro(buf, t.cs())));
 					MathData ar;
 					mathed_parse_cell(ar, '[' + opt + ']', mode_);
 					cell->append(ar);
@@ -1928,23 +1938,6 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 			cell->push_back(createInsetMath(t.cs(), buf));
 			parse2(cell->back(), FLAG_ITEM, mode, false);
 		}
-
-		// Disabled
-		else if (1 && t.cs() == "ar") {
-			auto_ptr<InsetMathXYArrow> p(new InsetMathXYArrow);
-			// try to read target
-			parse(p->cell(0), FLAG_OTPTION, mode);
-			// try to read label
-			if (nextToken().cat() == catSuper || nextToken().cat() == catSub) {
-				p->up_ = nextToken().cat() == catSuper;
-				getToken();
-				parse(p->cell(1), FLAG_ITEM, mode);
-				//lyxerr << "read label: " << p->cell(1) << endl;
-			}
-
-			cell->push_back(MathAtom(p.release()));
-			//lyxerr << "read cell: " << cell << endl;
-		}
 #endif
 
 		else if (t.cs() == "lyxmathsym") {
@@ -2007,7 +2000,10 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 							new InsetMathBig(t.cs(), delim)));
 					else {
 						cell->push_back(createInsetMath(t.cs(), buf));
-						putback();
+						// For some reason delim.empty()
+						// is always false here
+						if (delim.at(0))
+							putback();
 					}
 				}
 
@@ -2105,7 +2101,7 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 				}
 				if (!is_unicode_symbol) {
 					MathAtom at = is_user_macro ?
-						MathAtom(new MathMacro(buf, t.cs()))
+						MathAtom(new InsetMathMacro(buf, t.cs()))
 						: createInsetMath(t.cs(), buf);
 					InsetMath::mode_type m = mode;
 					//if (m == InsetMath::UNDECIDED_MODE)
