@@ -16,8 +16,11 @@
 #include "MathData.h"
 #include "MathExtern.h"
 
-#include "support/textutils.h"
+#include "TexRow.h"
+
 #include "support/docstring.h"
+#include "support/RefChanger.h"
+#include "support/textutils.h"
 
 #include <algorithm>
 #include <cstring>
@@ -128,16 +131,7 @@ WriteStream::WriteStream(otexrowstream & os, bool fragile, bool latex,
 	  output_(output), insidemacro_(false), pendingspace_(false),
 	  pendingbrace_(false), textmode_(false), locked_(0), ascii_(0),
 	  canbreakline_(true), mathsout_(false), ulemcmd_(NONE), line_(0),
-	  encoding_(encoding)
-{}
-
-
-WriteStream::WriteStream(otexrowstream & os)
-	: os_(os), fragile_(false), firstitem_(false), latex_(false),
-	  output_(wsDefault), insidemacro_(false), pendingspace_(false),
-	  pendingbrace_(false), textmode_(false), locked_(0), ascii_(0),
-	  canbreakline_(true), mathsout_(false), ulemcmd_(NONE), line_(0),
-	  encoding_(0)
+	  encoding_(encoding), row_entry_(TexRow::row_none)
 {}
 
 
@@ -186,26 +180,17 @@ void WriteStream::asciiOnly(bool ascii)
 }
 
 
-void WriteStream::pushRowEntry(TexRow::RowEntry entry)
+Changer WriteStream::changeRowEntry(TexRow::RowEntry entry)
 {
-	outer_row_entries_.push_back(entry);
-}
-
-
-void WriteStream::popRowEntry()
-{
-	if (!outer_row_entries_.empty())
-		outer_row_entries_.pop_back();
+	return make_change(row_entry_, entry);
 }
 
 
 bool WriteStream::startOuterRow()
 {
-	size_t n = outer_row_entries_.size();
-	if (n > 0)
-		return texrow().start(outer_row_entries_[n - 1]);
-	else
+	if (TexRow::isNone(row_entry_))
 		return false;
+	return texrow().start(row_entry_);
 }
 
 
@@ -310,7 +295,7 @@ void MathStream::defer(string const & s)
 
 
 docstring MathStream::deferred() const
-{ 
+{
 	return deferred_.str();
 }
 
@@ -400,7 +385,7 @@ void HtmlStream::defer(string const & s)
 
 
 docstring HtmlStream::deferred() const
-{ 
+{
 	return deferred_.str();
 }
 
