@@ -30,7 +30,7 @@ namespace {
 template<typename OFStream>
 bool doOpenFileWrite(OFStream & ofs, FileName const & fname)
 {
-	ofs.open(fname.toFilesystemEncoding().c_str());
+	ofs.open(fname.toSafeFilesystemEncoding(os::CREATE).c_str());
 	if (ofs)
 		return true;
 	docstring const file = fname.displayName(50);
@@ -40,7 +40,7 @@ bool doOpenFileWrite(OFStream & ofs, FileName const & fname)
 	return false;
 }
 
-}
+} // namespace
 
 
 bool openFileWrite(ofstream & ofs, FileName const & fname)
