@@ -28,7 +28,7 @@ class ios_base;
 template<typename CharT, typename Traits> class basic_streambuf;
 typedef basic_streambuf<char, char_traits<char> > streambuf;
 
-}
+} // namespace std
 #endif
 
 
@@ -110,7 +110,7 @@ namespace Debug {
 
 	/// Return number of levels
 	int levelCount();
- 
+
 	/// A function to convert debug level string names numerical values
 	Type value(std::string const & val);
 
@@ -144,7 +144,7 @@ class LyXErr
 public:
 	LyXErr(): dt_(Debug::NONE), stream_(0), enabled_(true),
 	          second_stream_(0), second_enabled_(false) {}
-	
+
 	/// Disable the stream completely
 	void disable();
 	/// Enable the stream after a possible call of disable()
@@ -165,7 +165,7 @@ public:
 	/// Returns second stream
 	std::ostream & secondStream() { return *second_stream_; }
 	/// Sets second stream
-	void setSecondStream(std::ostream * os) 
+	void setSecondStream(std::ostream * os)
 		{ second_enabled_ = (second_stream_ = os); }
 	/// Is the second stream is enabled?
 	bool secondEnabled() { return second_enabled_; }
@@ -202,6 +202,10 @@ LyXErr & operator<<(LyXErr &, int);
 LyXErr & operator<<(LyXErr &, unsigned int);
 LyXErr & operator<<(LyXErr &, long);
 LyXErr & operator<<(LyXErr &, unsigned long);
+#ifdef LYX_USE_LONG_LONG
+LyXErr & operator<<(LyXErr &, long long);
+LyXErr & operator<<(LyXErr &, unsigned long long);
+#endif
 LyXErr & operator<<(LyXErr &, double);
 LyXErr & operator<<(LyXErr &, std::string const &);
 LyXErr & operator<<(LyXErr &, docstring const &);
