@@ -97,7 +97,7 @@ BOOL terminate_handler(DWORD event)
 	return FALSE;
 }
 
-} // namespace anon
+} // namespace
 
 void init(int argc, char ** argv[])
 {
@@ -168,8 +168,8 @@ void init(int argc, char ** argv[])
 		*argv = newargv;
 	} else {
 		lyxerr << "LyX warning: Cannot make a copy of "
-			"command line arguments!"
-			<< endl;
+		          "command line arguments!"
+		       << endl;
 	}
 #endif
 
@@ -583,10 +583,13 @@ bool autoOpenFile(string const & filename, auto_open_mode const mode,
 		setEnv("TEXFONTS", newtexfonts);
 	}
 
+	QString const wname = toqstr(filename);
+
 	// reference: http://msdn.microsoft.com/en-us/library/bb762153.aspx
-	char const * action = (mode == VIEW) ? "open" : "edit";
-	bool success = reinterpret_cast<int>(ShellExecute(NULL, action,
-		to_local8bit(from_utf8(filename)).c_str(), NULL, NULL, 1)) > 32;
+	wchar_t const * action = (mode == VIEW) ? L"open" : L"edit";
+	bool success = reinterpret_cast<intptr_t>(ShellExecuteW(NULL, action,
+			reinterpret_cast<wchar_t const *>(wname.utf16()),
+			NULL, NULL, 1)) > 32;
 
 	if (!path.empty() && !lyxrc.texinputs_prefix.empty()) {
 		setEnv("TEXINPUTS", oldtexinputs);
