@@ -21,13 +21,16 @@ namespace Update {
 		/// Recenter the screen around the cursor if is found outside the
 		/// visible area.
 		FitCursor = 1,
-		/// Force a full screen metrics update.
+		/// Force a full screen metrics update and a full draw.
 		Force = 2,
+		/// Force a full redraw (but no metrics computations)
+		ForceDraw = 4,
 		/// Try to rebreak only the current paragraph metrics.
-		SinglePar = 4,
+		/// (currently ignored!)
+		SinglePar = 8,
 		/// Only the inset decorations need to be redrawn, no text metrics
 		/// update is needed.
-		Decoration = 8
+		Decoration = 16
 	};
 
 inline flags operator|(flags const f, flags const g)
@@ -40,6 +43,11 @@ inline flags operator&(flags const f, flags const g)
 	return static_cast<flags>(int(f) & int(g));
 }
 
+inline flags operator~(flags const f)
+{
+	return static_cast<flags>(~int(f));
+}
+
 } // namespace Update
 
 } // namespace lyx
